/*
 * File:        k30_emeter.h
 * Purpose:     Definitions for the Kinetis K40 power meter board
 *
 * Notes:
 */

#ifndef __K30_EMETER_H__
#define __K30_EMETER_H__

#include "fll.h"
#include "gpio.h"

/* Define for the MCU on the K40 emeter-board */
#define CPU_MK40X256VMD100

/* Core Clock selection */
#define CORE_CLK_MHZ        48          /* core clock in MHz (only 24,48,72) */

/* User IO defines */
#define LED_1               PIN_10
#define LED_2               PIN_11
#define LED_3               PIN_12

#define SW1                 PIN_0
#define VCC_PRESENT         PIN_2
#define TAMPER1             PIN_2
#define TAMPER2             PIN_4

/* SCI mode definitions */
#define FREEMASTER_MODE             0
#define BASIC_MODE                  1
#define SCI1_MODE                   FREEMASTER_MODE

/* Peripheral Clock Self-selection */
#if CORE_CLK_MHZ == 24
  #define PSCLK                     2   /* CORE_CLK_MHZ/4 */
  #define BUSCLK                    32768*732   /* Core frequency 23986176 Hz */
  #define ADIV                      2
  #define FLL_MODULE_CONFIG         FLL_MODULE_CONFIG_FEE_24MHZ
#endif      
#if CORE_CLK_MHZ == 48
  #define PSCLK                     3   /* CORE_CLK_MHZ/8 */
  #define BUSCLK                    32768*1464   /* Core frequency 47972352 Hz */
  #define ADIV                      2
  #define FLL_MODULE_CONFIG         FLL_MODULE_CONFIG_FEE_48MHZ
#endif      
#if CORE_CLK_MHZ == 72
  #define PSCLK                     4   /* CORE_CLK_MHZ/16 */
  #define BUSCLK                    32768*2197   /* Core frequency 71991296 Hz */
  #define ADIV                      4
  #define FLL_MODULE_CONFIG         FLL_MODULE_CONFIG_FEE_72MHZ
#endif      

/* Constants for zero-cross detection */
#define ZCD_CYCLES    5    /* Number of ZCD cycles (2^n) */
#define MIN_FREQ      (unsigned char)45   /* Minimum frequency for detection [Hz] */
#define SWING_FREQ    (unsigned short)110 /* Min. swing freq. to detection (>100Hz) */   
#define FOFFS         4    /* frequency offset */

/* Maximum overflow of MTIM3 to minimum frequency recognition for ZCD */
#define MAXMTIM16b    (unsigned short)(((((unsigned long long)(BUSCLK)>>16)<<ZCD_CYCLES)>>PSCLK)/(MIN_FREQ-2))

/* Minimum Zero-cross clock definitions (BUSCLK/PSCLK) */
#define MINZCDCLK     (unsigned long)(((unsigned long long)(BUSCLK)>>PSCLK)/SWING_FREQ)

/* Zero-cross clock definitions (BUSCLK*1000/PSCLK) - in thousandth MHz */
#define ZCDCLK        (unsigned long long)((((unsigned long long)(BUSCLK)*1000)<<ZCD_CYCLES)>>PSCLK)

/****************************************************************************** 
                      Specific peripherals defines 
******************************************************************************/ 
#define ADC_MODULE_16B_IREF_HWTRG_CONFIG_U                                    \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(ADIV))|              \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(0)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              SET(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(1)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  SET(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              CLR(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(1)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|CLR(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(2)),                    \
/* PGA    */  0l,                                                             \
}

#define ADC_MODULE_16B_IREF_HWTRG_CONFIG_I                                    \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(ADIV))|              \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(0)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              SET(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(1)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  SET(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              CLR(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(1)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|SET(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(2)),                    \
/* PGA    */  SET(ADC_PGA_PGAEN_MASK)|SET(ADC_PGA_PGAG(5)),                   \
}

#define PDB_MODULE_SWTRG_CONFIG2(presc,mult,modulo,idly)                      \
(tPDB){                                                                       \
/* SC     */  SET(PDB_SC_LDMOD(0))|CLR(PDB_SC_PDBEIE_MASK)|                   \
              CLR(PDB_SC_SWTRIG_MASK)|CLR(PDB_SC_DMAEN_MASK)|presc|           \
              SET(PDB_SC_TRGSEL(15))|SET(PDB_SC_PDBEN_MASK)|                  \
              SET(PDB_SC_PDBIE_MASK)|mult|SET(PDB_SC_CONT_MASK)|              \
              SET(PDB_SC_LDOK_MASK),                                          \
/* MOD    */  modulo,                                                         \
/* IDLY   */  idly,                                                           \
}

#define SIM_MODULE_CONFIG /* RTC,LCD,ADC0,ADC1,LPTMR and ports clocked */     \
(tSIM){                                                                       \
/* SOPT1   */ SIM_SOPT1_OSC32KSEL_MASK,                                       \
/* SOPT2   */ SIM_SOPT2_MCGCLKSEL_MASK,                                       \
/* SOPT4   */ 0l,                                                             \
/* SOPT5   */ 0l,                                                             \
/* SOPT6   */ 0l,                                                             \
/* SOPT7   */ 0l,                                                             \
/* SCGC1   */ 01,                                                             \
/* SCGC2   */ 0l,                                                             \
/* SCGC3   */ SIM_SCGC3_ADC1_MASK|SIM_SCGC3_SLCD_MASK,                        \
/* SCGC4   */ SIM_SCGC4_VREF_MASK,                                            \
/* SCGC5   */ SIM_SCGC5_PORTA_MASK|                                           \
              SIM_SCGC5_PORTC_MASK|SIM_SCGC5_PORTD_MASK|SIM_SCGC5_PORTE_MASK, \
/* SCGC6   */ SIM_SCGC6_RTC_MASK|SIM_SCGC6_PDB_MASK|SIM_SCGC6_PIT_MASK|       \
              SIM_SCGC6_ADC0_MASK|SIM_SCGC6_FTFL_MASK|SIM_SCGC6_DMAMUX_MASK,  \
/* SCGC7   */ SIM_SCGC7_DMA_MASK,                                             \
/* CLKDIV1 */ SIM_CLKDIV1_OUTDIV1(0)|SIM_CLKDIV1_OUTDIV2(0)|                  \
              SIM_CLKDIV1_OUTDIV3(0)|SIM_CLKDIV1_OUTDIV4(1),                  \
/* CLKDIV2 */ SIM_CLKDIV2_I2SDIV(1)|SIM_CLKDIV2_I2SFRAC(0)|                   \
              SIM_CLKDIV2_USBDIV(0)|SIM_CLKDIV2_USBFRAC_MASK,                 \
}
//|SIM_SCGC3_SLCD_MASK

#define TPM_MODULE_SYSCLK_EDGEALIGN_CONFIG                                    \
(tTPM)  {                                                                     \
/* SC       */ SET(FTM_SC_TOIE_MASK)|CLR(FTM_SC_CPWMS_MASK)|                  \
               SET(FTM_SC_CLKS(1))|SET(FTM_SC_PS(PSCLK)),                     \
}

#define RTC_MODULE_CONFIG2                                                     \
(tRTC){                                                                       \
/* TSR    */ 0l,                  /* value not used by the RTC_Init()     */  \
/* TPR    */ 0l,                  /* value not used by the RTC_Init()     */  \
/* TAR    */ 1l,                                                              \
/* TCR    */ RTC_TCR_TCR(0)|RTC_TCR_CIR(1),                                   \
/* CR     */ RTC_CR_OSCE_MASK,                                                \
/* SR     */ RTC_SR_TCE_MASK,                                                 \
/* LR     */ RTC_LR_SRL_MASK|RTC_LR_CRL_MASK|RTC_LR_TCL_MASK,                 \
/* WAR    */ RTC_WAR_CCRW_MASK|RTC_WAR_LRW_MASK|RTC_WAR_SRW_MASK|             \
             RTC_WAR_CRW_MASK|RTC_WAR_TCRW_MASK|RTC_WAR_TARW_MASK|            \
             RTC_WAR_TPRW_MASK|RTC_WAR_TSRW_MASK,                             \
/* RAR    */ RTC_RAR_CCRR_MASK|RTC_RAR_LRR_MASK|RTC_RAR_SRR_MASK|             \
             RTC_RAR_CRR_MASK|RTC_RAR_TCRR_MASK|RTC_RAR_TARR_MASK|            \
             RTC_RAR_TPRR_MASK|RTC_RAR_TSRR_MASK                              \
}


#define PORT_MODULE_BUTTON_IRQ_MODE2                                          \
(tPORT){        /* GPIO mode, pull-up enabled, IRQ on both edges */           \
/* PCR      */ PORT_PCR_MUX(1)|PORT_PCR_PS_MASK|PORT_PCR_PE_MASK|             \
               PORT_PCR_PFE_MASK|PORT_PCR_IRQC(11),                           \
/* DFER     */ PORT_DFER_DFE_MASK,                                            \
/* DFCR     */ 0l,                                                            \
/* DFWR     */ PORT_DFWR_FILT(10),                                            \
}

#define PORT_MODULE_ALT6_MODE2                     /* ALT6 push-pull mode */  \
(tPORT){                                                                      \
/* PCR      */ PORT_PCR_MUX(6),                                               \
/* DFER     */ 0l,                                                            \
/* DFCR     */ 0l,                                                            \
/* DFER     */ 0l,                                                            \
}

#endif /* __K30_EMETER_H__ */