/*****************************************************************************
 * (c) Copyright 2010, Freescale Semiconductor Inc.
 * ALL RIGHTS RESERVED.
 ***************************************************************************//*!
 * @file      adc.c
 * @author    R55013
 * @version   1.0.4.0
 * @date      Mar-8-2011
 * @brief     ADC driver implementation.
 * @par       Driver example:
 * @include   ADC1->test\ADC1->test.c 
 * @par       Interrupt vector definition(s):
 * @include   ADC1->test\isr.h		
 ******************************************************************************/
#include "common.h"
#include "adc.h"

/******************************************************************************
 * data type definitions                                                      *
 ******************************************************************************/
static ADC_CALLBACK volatile pCallbackADC0=NULL;
static ADC_CALLBACK volatile pCallbackADC1=NULL;
static ADC_CALLBACK volatile pCallbackADC2=NULL;
static ADC_CALLBACK volatile pCallbackADC3=NULL;

/******************************************************************************
 * interrupt functions prototype                                              *
 ******************************************************************************/
void ADC0_IRQHandler (void);
void ADC1_IRQHandler (void);
void ADC2_IRQHandler (void);
void ADC3_IRQHandler (void);


/******************************************************************************
 * Public functions definitions                                               *
 ******************************************************************************/
#pragma diag_suppress=Pa082
void ADC_Init  (ADC_Type * pAdc, tADC adc, tADC_CH CHA, tADC_CH CHB, uint8 ip, ADC_CALLBACK pCallback)
{
  if (pCallback != NULL && pAdc == ADC0) { pCallbackADC0 = pCallback; NVIC_EnableIRQ(ADC0_IRQn); }
  else if (pCallback != NULL && pAdc == ADC1) { pCallbackADC1 = pCallback; NVIC_EnableIRQ(ADC1_IRQn); }
  else if (pCallback != NULL && pAdc == ADC2) { pCallbackADC2 = pCallback; NVIC_EnableIRQ(ADC2_IRQn); }
  else if (pCallback != NULL && pAdc == ADC3) { pCallbackADC3 = pCallback; NVIC_EnableIRQ(ADC3_IRQn); }
  
  pAdc->CFG1 = adc.CFG1;
  pAdc->CFG2 = adc.CFG2;
  pAdc->CV1  = adc.CV1;
  pAdc->CV2  = adc.CV2;
  pAdc->SC2  = adc.SC2;
  pAdc->SC3  = adc.SC3;
  pAdc->PGA  = adc.PGA;
  pAdc->SC1[0] = CHA.SC1;
  pAdc->SC1[1] = CHB.SC1;  
}


#define TWO_POWER_N_SAMPLES  6  /* select carefully - COP timer not updated */
int ADC_ExecCalib ( ADC_Type * pAdc , tADC adc, tADC_CALIB *pCALIB)
{
  long tmp=0;
  unsigned long tmp_1[7]={0,0,0,0,0,0,0}, tmp_2[7]={0,0,0,0,0,0,0}; 
  register uint16 i=0, numLoops = ((uint16)1<<TWO_POWER_N_SAMPLES);
  
  pAdc->CFG1 = adc.CFG1;
  pAdc->CFG2 = adc.CFG2;
  pAdc->SC2  = (adc.SC2 & (~ADC_SC2_ADTRG_MASK));
  
  /* calibration loop                                                       */
  while ((i++) < numLoops)
  { 
    /* The input channel, conversion mode continuous function, compare      */
    /* function, resolution mode, and differential/single-ended mode are    */
    /* all ignored during the calibration function.                         */
    pAdc->SC1[0] = 0x1f;
    pAdc->SC3  = (adc.SC3 & (~ADC_SC3_CAL_MASK));
    pAdc->SC3 |= ADC_SC3_CAL_MASK;
    while (!pAdc->SC1[CHANNEL_A] & ADC_SC1_COCO_MASK); //ADC ready?
    if (pAdc->SC3 & ADC_SC3_CALF_MASK) { pAdc->SC3 |= ADC_SC3_CALF_MASK; return 1; }
    
    tmp+= (short int)pAdc->OFS;
    
    tmp_1[0]+=(unsigned long)pAdc->CLP0;
    tmp_1[1]+=(unsigned long)pAdc->CLP1;
    tmp_1[2]+=(unsigned long)pAdc->CLP2;
    tmp_1[3]+=(unsigned long)pAdc->CLP3;
    tmp_1[4]+=(unsigned long)pAdc->CLP4;
    tmp_1[5]+=(unsigned long)pAdc->CLPS;
    tmp_1[6]+=(unsigned long)pAdc->CLPD;
    
    tmp_2[0]+=(unsigned long)pAdc->CLM0;
    tmp_2[1]+=(unsigned long)pAdc->CLM1;
    tmp_2[2]+=(unsigned long)pAdc->CLM2;
    tmp_2[3]+=(unsigned long)pAdc->CLM3;
    tmp_2[4]+=(unsigned long)pAdc->CLM4;
    tmp_2[5]+=(unsigned long)pAdc->CLMS;
    tmp_2[6]+=(unsigned long)pAdc->CLMD;
  }
  pCALIB->OFS  = tmp >> TWO_POWER_N_SAMPLES;
  
  pCALIB->CLP[0] = tmp_1[0] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLP[1] = tmp_1[1] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLP[2] = tmp_1[2] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLP[3] = tmp_1[3] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLP[4] = tmp_1[4] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLPS   = tmp_1[5] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLPD   = tmp_1[6] >> TWO_POWER_N_SAMPLES;    
  
  tmp = tmp_1[0]+tmp_1[1]+tmp_1[2]+tmp_1[3]+tmp_1[4]+tmp_1[5];
  pCALIB->PG  = ((tmp>>(1+TWO_POWER_N_SAMPLES))|0x8000);
  
  pCALIB->CLM[0] = tmp_2[0] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLM[1] = tmp_2[1] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLM[2] = tmp_2[2] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLM[3] = tmp_2[3] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLM[4] = tmp_2[4] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLMS   = tmp_2[5] >> TWO_POWER_N_SAMPLES;  
  pCALIB->CLMD   = tmp_2[6] >> TWO_POWER_N_SAMPLES;    
  
  tmp = tmp_2[0]+tmp_2[1]+tmp_2[2]+tmp_2[3]+tmp_2[4]+tmp_2[5];
  pCALIB->MG  = ((tmp>>(1+TWO_POWER_N_SAMPLES))|0x8000);
  
  ADC_SaveCalib (pAdc, pCALIB);     /* update calibration registers         */

  return 0; 
}


void ADC_SaveCalib (ADC_Type * pAdc, tADC_CALIB *pCALIB)
{
  pAdc->CLP0 = pCALIB->CLP[0]; 
  pAdc->CLP1 = pCALIB->CLP[1];
  pAdc->CLP2 = pCALIB->CLP[2];
  pAdc->CLP3 = pCALIB->CLP[3];
  pAdc->CLP4 = pCALIB->CLP[4];
  pAdc->CLPS = pCALIB->CLPS;
  pAdc->CLPD = pCALIB->CLPD;  
  pAdc->CLM0 = pCALIB->CLM[0];
  pAdc->CLM1 = pCALIB->CLM[1];
  pAdc->CLM2 = pCALIB->CLM[2];
  pAdc->CLM3 = pCALIB->CLM[3];
  pAdc->CLM4 = pCALIB->CLM[4];
  pAdc->CLMS = pCALIB->CLMS;
  pAdc->CLMD = pCALIB->CLMD;
  pAdc->PG   = pCALIB->PG; 
  pAdc->MG   = pCALIB->MG; 
  pAdc->OFS  = pCALIB->OFS;     
}

/******************************************************************************
 * interrupt functions definitions                                            *
 ******************************************************************************/
void ADC0_IRQHandler (void) 
{ 
  if (pCallbackADC0 != (ADC_CALLBACK)NULL) 
  { 
    if (ADC0->SC1[0] & ADC_SC1_COCO_MASK) { pCallbackADC0 (CHA_CALLBACK, ADC0->R[0]); }
    if (ADC0->SC1[1] & ADC_SC1_COCO_MASK) { pCallbackADC0 (CHB_CALLBACK, ADC0->R[1]); }
  }
}

void ADC1_IRQHandler (void) 
{ 
  if (pCallbackADC1 != (ADC_CALLBACK)NULL) 
  { 
    if (ADC1->SC1[0] & ADC_SC1_COCO_MASK) { pCallbackADC1 (CHA_CALLBACK, ADC1->R[0]); }
    if (ADC1->SC1[1] & ADC_SC1_COCO_MASK) { pCallbackADC1 (CHB_CALLBACK, ADC1->R[1]); }
  }
}

void ADC2_IRQHandler (void) 
{ 
  if (pCallbackADC2 != (ADC_CALLBACK)NULL) 
  { 
    if (ADC2->SC1[0] & ADC_SC1_COCO_MASK) { pCallbackADC2 (CHA_CALLBACK, ADC2->R[0]); }
    if (ADC2->SC1[1] & ADC_SC1_COCO_MASK) { pCallbackADC2 (CHB_CALLBACK, ADC2->R[1]); }
  }
}

void ADC3_IRQHandler (void) 
{ 
  if (pCallbackADC3 != (ADC_CALLBACK)NULL) 
  { 
    if (ADC3->SC1[0] & ADC_SC1_COCO_MASK) { pCallbackADC3 (CHA_CALLBACK, ADC3->R[0]); }
    if (ADC3->SC1[1] & ADC_SC1_COCO_MASK) { pCallbackADC3 (CHB_CALLBACK, ADC3->R[1]); }
  }
}
/******************************************************************************
 * End of module                                                              *
 ******************************************************************************/