/*****************************************************************************
 * (c) Copyright 2010, Freescale Semiconductor Inc.
 * ALL RIGHTS RESERVED.
 ***************************************************************************//*!
 * @file      adc.h
 * @author    R55013
 * @version   1.0.4.0
 * @date      Mar-8-2011
 * @brief     ADC driver header file.
 ******************************************************************************/
#ifndef __ADC_H
#define __ADC_H 
#include <MK10F12.H>

/******************************************************************************
 * ADC calibration data structure definition                                  *
 ******************************************************************************/
typedef struct { uint32 OFS, PG, MG, CLPD, CLPS, CLP[5], CLMD, CLMS, CLM[5]; } tADC_CALIB;
 
/******************************************************************************
 * definition configuration structure                                         *
 ******************************************************************************/
typedef struct { uint32 CFG1, CFG2, CV1, CV2, SC2, SC3, PGA; } tADC;
typedef struct { uint32 SC1; } tADC_CH;

/******************************************************************************
 * command definitions                                                        *
 ******************************************************************************/
/* analog channels definition for use with ADC for use with GPIO macros                              */
#define DAD0          (uint8)0
#define DAD1          (uint8)1
#define DAD2          (uint8)2
#define DAD3          (uint8)3
#define AD04          (uint8)4
#define AD05          (uint8)5
#define AD06          (uint8)6
#define AD07          (uint8)7
#define AD08          (uint8)8
#define AD09          (uint8)9
#define AD10          (uint8)10
#define AD11          (uint8)11
#define AD12          (uint8)12
#define AD13          (uint8)13
#define AD14          (uint8)14
#define AD15          (uint8)15
#define AD16          (uint8)16
#define AD17          (uint8)17
#define AD18          (uint8)18
#define AD19          (uint8)19
#define AD20          (uint8)20
#define AD21          (uint8)21
#define AD22          (uint8)22
#define AD23          (uint8)23
#define AD24          (uint8)24
#define AD25          (uint8)25
#define AD26          (uint8)26
#define AD27          (uint8)27
#define AD28          (uint8)28
#define AD29          (uint8)29
#define AD30          (uint8)30
#define AD31          (uint8)31

#define ADIV_1  0
#define ADIV_2  1
#define ADIV_4  2
#define ADIV_8  3

#define CHANNEL_A 0
#define CHANNEL_B 1

/******************************************************************************
 * Default ADC configurations                                                 *
 ******************************************************************************/
#define ADC_MODULE_16B_SWTRG_IREF_AVG32_PGAOFF_CONFIG                         \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(3))|                 \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(0)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              SET(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(0)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  CLR(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              CLR(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(1)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|CLR(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(3)),                    \
/* PGA    */  0l,                                                             \
}

#define ADC_MODULE_16B_SWTRG_XREF_AVG32_PGAOFF_CONFIG                         \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(3))|                 \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(0)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              SET(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(0)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  CLR(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              CLR(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(0)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|CLR(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(3)),                    \
/* PGA    */  0l,                                                             \
}

#define ADC_MODULE_16B_IREF_HWTRG_CONFIG_U                                    \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(ADIV_4))|              \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(0)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              SET(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(1)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  SET(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              CLR(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(1)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|CLR(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(2)),                    \
/* PGA    */  0l,                                                             \
}

#define ADC_MODULE_16B_IREF_HWTRG_CONFIG_I                                    \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(ADIV))|              \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(0)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              SET(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(1)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  SET(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              CLR(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(1)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|SET(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(2)),                    \
/* PGA    */  SET(ADC_PGA_PGAEN_MASK)|SET(ADC_PGA_PGAG(5)),                   \
}

#define ADC_MODULE_16B_HWTRG_IREF_AVG16_PGAOFF_CONFIG                         \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(0))|                 \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(1)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              CLR(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(0)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  SET(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              CLR(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(1)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|CLR(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(2)),                    \
/* PGA    */  0l,                                                             \
}

#define ADC_MODULE_16B_HWTRG_DMA_IREF_AVG16_PGAOFF_CONFIG                     \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(0))|                 \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(1)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              CLR(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(0)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  SET(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              SET(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(1)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|CLR(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(2)),                    \
/* PGA    */  0l,                                                             \
}

#define ADC_MODULE_16B_SWTRG_DMA_IREF_AVG16_PGAOFF_CONFIG                     \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(0))|                 \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(1)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              CLR(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(0)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  CLR(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              SET(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(1)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|CLR(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(2)),                    \
/* PGA    */  0l,                                                             \
}

#define ADC_MODULE_16B_HWTRG_XREF_AVG32_PGAOFF_CONFIG                         \
(tADC){                                                                       \
/* CFG1   */  CLR(ADC_CFG1_ADLPC_MASK)|SET(ADC_CFG1_ADIV(3))|                 \
              SET(ADC_CFG1_ADLSMP_MASK)|SET(ADC_CFG1_MODE(3))|                \
              SET(ADC_CFG1_ADICLK(0)),                                        \
/* CFG2   */  CLR(ADC_CFG2_MUXSEL_MASK)|CLR(ADC_CFG2_ADACKEN_MASK)|           \
              SET(ADC_CFG2_ADHSC_MASK)|SET(ADC_CFG2_ADLSTS(0)),               \
/* CV1    */  0l,                                                             \
/* CV2    */  0l,                                                             \
/* SC2    */  SET(ADC_SC2_ADTRG_MASK)|CLR(ADC_SC2_ACFE_MASK)|                 \
              CLR(ADC_SC2_ACFGT_MASK)|CLR(ADC_SC2_ACREN_MASK)|                \
              CLR(ADC_SC2_DMAEN_MASK)|SET(ADC_SC2_REFSEL(0)),                 \
/* SC3    */  CLR(ADC_SC3_CAL_MASK)|CLR(ADC_SC3_ADCO_MASK)|                   \
              SET(ADC_SC3_AVGE_MASK)|SET(ADC_SC3_AVGS(3)),                    \
/* PGA    */  0l,                                                             \
}

#define ADC_CH_SE_IRQ_CONFIG(inp)     /* single ended, interrupt enabled  */  \
(tADC_CH){                                                                    \
/* SC1    */ SET(ADC_SC1_AIEN_MASK)|CLR(ADC_SC1_DIFF_MASK)|inp,               \
}

#define ADC_CH_SE_POLL_CONFIG(inp)    /* single ended, polling mode       */  \
(tADC_CH){                                                                    \
/* SC1    */ CLR(ADC_SC1_AIEN_MASK)|CLR(ADC_SC1_DIFF_MASK)|inp,               \
}

#define ADC_CH_DI_IRQ_CONFIG(inp)     /* differential, interrupt enabled  */  \
(tADC_CH){                                                                    \
/* SC1    */ SET(ADC_SC1_AIEN_MASK)|SET(ADC_SC1_DIFF_MASK)|inp,               \
}

#define ADC_CH_DI_POLL_CONFIG(inp)    /* differential, polling mode       */  \
(tADC_CH){                                                                    \
/* SC1    */ CLR(ADC_SC1_AIEN_MASK)|SET(ADC_SC1_DIFF_MASK)|inp,               \
}

/******************************************************************************
 * data type definitions                                                      *
 ******************************************************************************/
typedef enum {CHA_CALLBACK=1, CHB_CALLBACK=2 } ADC_CALLBACK_TYPE;
typedef void (*ADC_CALLBACK)(ADC_CALLBACK_TYPE type, int16 result);

/******************************************************************************
 * command definitions                                                        *
 ******************************************************************************/ 
/* ADC macro definitions                                                      */
#define ADC0_CHA_Start(inp) REG_SetVal(ADC0->SC1[0],ADC_SC1_ADCH_MASK,inp,ADC_SC1_ADCH_SHIFT)
#define ADC0_CHB_Start(inp) REG_SetVal(ADC0->SC1[1],ADC_SC1_ADCH_MASK,inp,ADC_SC1_ADCH_SHIFT)
#define ADC1_CHA_Start(inp) REG_SetVal(ADC1->SC1[0],ADC_SC1_ADCH_MASK,inp,ADC_SC1_ADCH_SHIFT)
#define ADC1_CHB_Start(inp) REG_SetVal(ADC1->SC1[1],ADC_SC1_ADCH_MASK,inp,ADC_SC1_ADCH_SHIFT)
#define ADC2_CHA_Start(inp) REG_SetVal(ADC2->SC1[0],ADC_SC1_ADCH_MASK,inp,ADC_SC1_ADCH_SHIFT)
#define ADC2_CHB_Start(inp) REG_SetVal(ADC2->SC1[1],ADC_SC1_ADCH_MASK,inp,ADC_SC1_ADCH_SHIFT)
#define ADC3_CHA_Start(inp) REG_SetVal(ADC3->SC1[0],ADC_SC1_ADCH_MASK,inp,ADC_SC1_ADCH_SHIFT)
#define ADC3_CHB_Start(inp) REG_SetVal(ADC3->SC1[1],ADC_SC1_ADCH_MASK,inp,ADC_SC1_ADCH_SHIFT)
/***************************************************************************//*!
 * @brief   Macro starts ADC conversion on given input.
 * @param   module  - ADC0|ADC1
 * @param   ch      - CHA|CHB
 * @param   inp     - AD0..AD31
 * @note    Implemented as inlined macro.
 ******************************************************************************/
#define ADC_Start(module,ch,inp) module##_##ch##_Start(inp)

#define ADC0_CHA_Ready           (ADC0->SC1[0] & ADC_SC1_COCO_MASK)
#define ADC0_CHB_Ready           (ADC0->SC1[1] & ADC_SC1_COCO_MASK)
#define ADC1_CHA_Ready           (ADC1->SC1[0] & ADC_SC1_COCO_MASK)
#define ADC1_CHB_Ready           (ADC1->SC1[1] & ADC_SC1_COCO_MASK)
#define ADC2_CHA_Ready           (ADC2->SC1[0] & ADC_SC1_COCO_MASK)
#define ADC2_CHB_Ready           (ADC2->SC1[1] & ADC_SC1_COCO_MASK)
#define ADC3_CHA_Ready           (ADC3->SC1[0] & ADC_SC1_COCO_MASK)
#define ADC3_CHB_Ready           (ADC3->SC1[1] & ADC_SC1_COCO_MASK)
/***************************************************************************//*!
 * @brief   Macro returns ADC conversion status (COCO flag)
 * @param   module  - ADC0|ADC1
 * @param   ch      - CHA|CHB
 * @return  TRUE    - conversion completed
 * @return  FALSE   - conversion pending
 * @note    Implemented as inlined macro.
 ******************************************************************************/
 #define ADC_Ready(module,ch)     module##_##ch##_Ready
//#define ADC_Ready(module,ch)                                \
//{                                                               \
//  (##module##->SC1[##ch##] & ADC_SC1_COCO_MASK)
//}



#define ADC0_CHA_Read            (uint16)ADC0->R[0]
#define ADC0_CHB_Read            (uint16)ADC0->R[1]
#define ADC1_CHA_Read            (uint16)ADC1->R[0]
#define ADC1_CHB_Read            (uint16)ADC1->R[1]
#define ADC2_CHA_Read            (uint16)ADC2->R[0]
#define ADC2_CHB_Read            (uint16)ADC2->R[1]
#define ADC3_CHA_Read            (uint16)ADC3->R[0]
#define ADC3_CHB_Read            (uint16)ADC3->R[1]

/***************************************************************************//*!
 * @brief   Macro returns ADC result register value.
 * @param   module  - ADC0|ADC1
 * @param   ch      - CHA|CHB
 * @return  result register value
 * @note    Implemented as inlined macro.
 ******************************************************************************/
#define ADC_Read(module,ch)      module##_##ch##_Read

/* Function redefinition                                                      */
/***************************************************************************//*!
 * @brief   ADC initialization function.
 * @param   module  - ADC0|ADC1
 * @param   cfg     - ADC module configuration structure passed by value: 
 *                    ADC_MODULE_16B_SWTRG_IREF_AVG32_PGAOFF_CONFIG,
 *                    ADC_MODULE_16B_SWTRG_XREF_AVG32_PGAOFF_CONFIG,
 *                    ADC_MODULE_16B_HWTRG_IREF_AVG32_PGAOFF_CONFIG,
 *                    ADC_MODULE_16B_HWTRG_XREF_AVG32_PGAOFF_CONFIG.
 * @param   CHA     - ADC channel "A" configuration structure passed by value:
 *                    ADC_CH_SE_IRQ_CONFIG(inp),
 *                    ADC_CH_SE_POLL_CONFIG(inp),
 *                    ADC_CH_DI_IRQ_CONFIG(inp),
 *                    ADC_CH_DI_POLL_CONFIG(inp).
 *                    where inp is ADC input channel AD0..AD30, AD31(disabled) 
 * @param   CHB     - ADC channel "B" configuration structure passed by value:
 *                    ADC_CH_SE_IRQ_CONFIG(inp),
 *                    ADC_CH_SE_POLL_CONFIG(inp),
 *                    ADC_CH_DI_IRQ_CONFIG(inp),
 *                    ADC_CH_DI_POLL_CONFIG(inp).
 *                    where inp is ADC input channel AD0..AD30, AD31(disabled) 
 * @param   ip      - interrupt priority 0..15
 * @param   callback- pointer to ADC_CALLBACK function
 * @return  none
 * @note    Implemented as function call.
 ******************************************************************************/
//#define ADC_Init(module,cfg,CHA,CHB,ip,callback)  module##_##Init(cfg,CHA,CHB,ip,callback)

  
/******************************************************************************
 * public function prototypes                                                 *
 ******************************************************************************/    

/***************************************************************************//*!
 * @brief   ADC calibration function. Function performs calibration and updates
 *          calibration registers.
 * @param   module  - ADC0|ADC1
 * @param   cfg     - ADC module configuration structure passed by value: 
 *                    ADC_MODULE_16B_SWTRG_IREF_AVG32_PGAOFF_CONFIG,
 *                    ADC_MODULE_16B_SWTRG_XREF_AVG32_PGAOFF_CONFIG
 * @param   *pCALIB - pointer to tCALIB data structure.
 * @return  0 - SUCCESS 
 * @return  1 - ERROR
 * @note    Implemented as function call.
 ******************************************************************************/
extern int ADC_ExecCalib (ADC_Type * pAdc, tADC adc, tADC_CALIB *pCALIB);

/***************************************************************************//*!
 * @brief   Function updates calibration registers of the ADC.
 * @param   module  - ADC0|ADC1
 * @param   *pCALIB - pointer to tCALIB data structure.
 * @return  none
 * @note    Implemented as function call.
 ******************************************************************************/
extern void ADC_SaveCalib (ADC_Type * pAdc, tADC_CALIB *pCALIB);

extern void ADC_Init (ADC_Type * pAdc, tADC adc, tADC_CH CHA, tADC_CH CHB, uint8 ip, ADC_CALLBACK pCallback);


#endif /* __ADC_H */ 

