/******************************************************************************
* 
* Copyright (c) 2009 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      LCD.c
* @author    b19186
* @version   0.0.1.0
* @date      April-11-2011
* @brief     Driver for LCD
* 					
******************************************************************************/
#include <string.h>
#include "common.h"
#include "LCD.h"
#include "LCD_glass.h"

unsigned char LCD_buffer[_LCD_CHARNUM1+1];              // LCD buffer
unsigned char bLCD_CharPosition;
unsigned char *bpaPlace[ ] =
{ /* CharNa = SEGs: A,F,E,D, CharNb = SEGs: B,G,C,- */
 (unsigned char *)&Seg1_01,   // LCD16 --- Pin:6    
 (unsigned char *)&Seg2_01,   // LCD19 --- Pin:5   
 (unsigned char *)&Seg1_02,   // LCD32 --- Pin:21  
 (unsigned char *)&Seg2_02,   // LCD33 --- Pin:20    
 (unsigned char *)&Seg1_03,   // LCD30 --- Pin:23    
 (unsigned char *)&Seg2_03,   // LCD31 --- Pin:22   
 (unsigned char *)&Seg1_04,   // LCD28 --- Pin:25    
 (unsigned char *)&Seg2_04,   // LCD29 --- Pin:24    
 (unsigned char *)&Seg1_05,   // LCD26 --- Pin:27    
 (unsigned char *)&Seg2_05,   // LCD27 --- Pin:26    
 (unsigned char *)&Seg1_06,   // LCD24 --- Pin:29    
 (unsigned char *)&Seg2_06,   // LCD25 --- Pin:28    
 (unsigned char *)&Seg1_07,   // LCD22 --- Pin:31    
 (unsigned char *)&Seg2_07,   // LCD23 --- Pin:30    
 (unsigned char *)&Seg1_08,   // LCD20 --- Pin:33    
 (unsigned char *)&Seg2_08,   // LCD21 --- Pin:32   
 (unsigned char *)&Seg1_09,   // LCD17 --- Pin:35    
 (unsigned char *)&Seg2_09,   // LCD18 --- Pin:34    
 (unsigned char *)&Seg1_10,   // LCD4 --- Pin:8    
 (unsigned char *)&Seg2_10,   // LCD11 --- Pin:7    
 (unsigned char *)&Seg1_11,   // LCD10 --- Pin:10   
 (unsigned char *)&Seg2_11,   // LCD7 --- Pin:9   
 (unsigned char *)&Seg1_12,   // LCD15 --- Pin:12   
 (unsigned char *)&Seg2_12,   // LCD13 --- Pin:11   
 (unsigned char *)&Seg1_13,   // LCD6 --- Pin:14   
 (unsigned char *)&Seg2_13,   // LCD5 --- Pin:13  
 (unsigned char *)&Seg1_14,   // LCD9 --- Pin:16   
 (unsigned char *)&Seg2_14,   // LCD8 --- Pin:15   
 (unsigned char *)&Seg1_15,   // LCD9 --- Pin:16   
 (unsigned char *)&Seg2_15,   // LCD8 --- Pin:15   
};


/*
Codification from Ascci code 0 to Z   
This Display has 4COM/BackPlane. It requires 2 values to complete one character
':' = SPACE
*/
const unsigned char baAscii [ ] =
{
  N0_1, N0_2,                 //Char = 0,   offset=0
  N1_1, N1_2,                 //Char = 1,   offset=2
  N2_1, N2_2,                 //Char = 2,   offset=4
  N3_1, N3_2,                 //Char = 3,   offset=6
  N4_1, N4_2,                 //Char = 4,   offset=8
  N5_1, N5_2,                 //Char = 5,   offset=10
  N6_1, N6_2,                 //Char = 6,   offset=12
  N7_1, N7_2,                 //Char = 7,   offset=14
  N8_1, N8_2,                 //Char = 8,   offset=16
  N9_1, N9_2,                 //Char = 9,   offset=18
  SP_1, SP_2,                 //Char =  ,   offset=20
  CA_1, CA_2,                 //Char = A,   offset=22
  CB_1, CB_2,                 //Char = b,   offset=24
  CC_1, CC_2,                 //Char = c,   offset=26
  CD_1, CD_2,                 //Char = d,   offset=28
  CE_1, CE_2,                 //Char = E,   offset=30
  CF_1, CF_2                 //Char = F,   offset=32

 
/*  
 (!SEGG+ SEGE+ SEGF) ,( SEGB+ SEGD+ SEGC+ SEGA),      //Char = 0,   offset=0
 (!SEGG+!SEGE+!SEGF) ,( SEGB+!SEGD+ SEGC+!SEGA),      //Char = 1,   offset=2
 ( SEGG+ SEGE+!SEGF) ,( SEGB+ SEGD+!SEGC+ SEGA),      //Char = 2,   offset=4
 ( SEGG+!SEGE+!SEGF) ,( SEGB+ SEGD+ SEGC+ SEGA),      //Char = 3,   offset=6
 ( SEGG+!SEGE+ SEGF) ,( SEGB+!SEGD+ SEGC+!SEGA),      //Char = 4,   offset=8
 ( SEGG+!SEGE+ SEGF) ,(!SEGB+ SEGD+ SEGC+ SEGA),      //Char = 5,   offset=10
 ( SEGG+ SEGE+ SEGF) ,(!SEGB+ SEGD+ SEGC+ SEGA),      //Char = 6,   offset=12
 (!SEGG+!SEGE+!SEGF) ,( SEGB+!SEGD+ SEGC+ SEGA),      //Char = 7,   offset=14
 ( SEGG+ SEGE+ SEGF) ,( SEGB+ SEGD+ SEGC+ SEGA),      //Char = 8,   offset=16
 ( SEGG+!SEGE+ SEGF) ,( SEGB+ SEGD+ SEGC+ SEGA),      //Char = 9,   offset=18
 (!SEGG+!SEGE+!SEGF) ,(!SEGB+!SEGD+!SEGC+!SEGA),      //Char =  ,   offset=20
 ( SEGG+ SEGE+ SEGF) ,( SEGB+!SEGD+ SEGC+ SEGA),      //Char = A,   offset=22
 ( SEGG+ SEGE+ SEGF) ,(!SEGB+ SEGD+ SEGC+!SEGA),      //Char = b,   offset=24
 ( SEGG+ SEGE+!SEGF) ,(!SEGB+ SEGD+!SEGC+!SEGA),      //Char = c,   offset=26
 ( SEGG+ SEGE+!SEGF) ,( SEGB+ SEGD+ SEGC+!SEGA),      //Char = d,   offset=28
 ( SEGG+ SEGE+ SEGF) ,(!SEGB+ SEGD+!SEGC+ SEGA),      //Char = E,   offset=30
 ( SEGG+ SEGE+ SEGF) ,(!SEGB+!SEGD+!SEGC+ SEGA),      //Char = F,   offset=32
 ( SEGG+ SEGE+ SEGF) ,(!SEGB+ SEGD+ SEGC+ SEGA),      //Char = G,   offset=34
 ( SEGG+ SEGE+ SEGF) ,( SEGB+!SEGD+ SEGC+!SEGA),      //Char = H,   offset=36
 (!SEGG+!SEGE+!SEGF) ,( SEGB+!SEGD+ SEGC+!SEGA),      //Char = I,   offset=38
 (!SEGG+!SEGE+!SEGF) ,( SEGB+ SEGD+ SEGC+!SEGA),      //Char = J,   offset=40
 ( SEGG+ SEGE+ SEGF) ,( SEGB+!SEGD+ SEGC+!SEGA),      //Char = K,   offset=42
 (!SEGG+ SEGE+ SEGF) ,(!SEGB+ SEGD+!SEGC+!SEGA),      //Char = L,   offset=44
 (!SEGG+ SEGE+ SEGF) ,( SEGB+!SEGD+ SEGC+ SEGA),      //Char = M,   offset=46
 ( SEGG+ SEGE+!SEGF) ,(!SEGB+!SEGD+ SEGC+!SEGA),      //Char = N,   offset=48
 ( SEGG+ SEGE+!SEGF) ,(!SEGB+ SEGD+ SEGC+!SEGA),      //Char = O,   offset=50
 ( SEGG+ SEGE+ SEGF) ,( SEGB+!SEGD+!SEGC+ SEGA),      //Char = P,   offset=52
 ( SEGG+ SEGE+!SEGF) ,(!SEGB+ SEGD+ SEGC+!SEGA),      //Char = Q,   offset=54
 ( SEGG+ SEGE+!SEGF) ,(!SEGB+!SEGD+!SEGC+!SEGA),      //Char = R,   offset=56
 ( SEGG+!SEGE+ SEGF) ,(!SEGB+ SEGD+ SEGC+ SEGA),      //Char = S,   offset=58
 (!SEGG+ SEGE+ SEGF) ,(!SEGB+!SEGD+!SEGC+ SEGA),      //Char = T,   offset=60
 (!SEGG+ SEGE+ SEGF) ,( SEGB+ SEGD+ SEGC+!SEGA),      //Char = U,   offset=62
 (!SEGG+ SEGE+!SEGF) ,(!SEGB+ SEGD+ SEGC+!SEGA),      //Char = V,   offset=64
 (!SEGG+ SEGE+!SEGF) ,(!SEGB+ SEGD+ SEGC+!SEGA),      //Char = W,   offset=66
 ( SEGG+ SEGE+ SEGF) ,( SEGB+!SEGD+ SEGC+!SEGA),      //Char = X,   offset=68
 ( SEGG+ SEGE+ SEGF) ,( SEGB+!SEGD+!SEGC+!SEGA),      //Char = Y,   offset=70
 ( SEGG+ SEGE+!SEGF) ,( SEGB+ SEGD+!SEGC+ SEGA),      //Char = Z,   offset=72
*/
};

/***************************************************************************//*!
*
* @brief    Initialization of LCD   
*
* @param    nothing
* @return   nothing
*
******************************************************************************/
void vfnLCD_Init(void)
{
  MCG_C1     |= 0x02;                 // enable IRCLK
   
  LCD_PENL   = 0xF33FFFFF;            //enable LCD pins
  LCD_PENH   = 0x0000FFFF;
  LCD_BPENL  = 0x0000000F;            //enable LCD backplanes 0, 1, 2, 3

  LCD_WF3TO0 = WF3TOx_BACKPL;         //assign back planes 

  LCD_GCR    |=LCD_GCR_VSUPPLY(1);
  LCD_GCR    |=LCD_GCR_CPSEL_MASK;
  LCD_GCR    |=LCD_GCR_LADJ(0);       // !!!
  LCD_GCR    |=LCD_GCR_DUTY(3);
  LCD_GCR    |=LCD_GCR_SOURCE_MASK;   //select alt clock IRC
  LCD_GCR    |=LCD_GCR_LCLK(3);       //4
  
  LCD_GCR    |= LCD_GCR_LCDEN_MASK;   //Enable LCD
}
  
/***************************************************************************//*!
*
* @brief    Change of contrast of LCD   
*
* @param    value (0..15)
* @return   nothing
*
******************************************************************************/
void vfnLCD_Contrast (unsigned char lbContrast)
{
  lbContrast &= 0x0F;                                                                                                           
  LCD_GCR    |= LCD_GCR_RVEN_MASK;                                                                                       
  LCD_GCR    |= LCD_GCR_RVTRIM(lbContrast);                 
}
 
/******************************************************************************
                        Symbol mask for LCD segments 1..14
******************************************************************************/
const unsigned char symbol_mask[] = {
    0x00, 0x80,       // Char 1b, Char 1a
    0x00, 0x80,       // Char 2b, Char 2a
    0x00, 0x80,       // Char 3b, Char 3a
    0x00, 0x80,       // Char 4b, Char 4a
    0x00, 0x80,       // Char 5b, Char 5a
    0x00, 0x80,       // Char 6b, Char 6a
    0x00, 0x80,       // Char 7b, Char 7a
    0x00, 0x80,       // Char 8b, Char 8a
    0x00, 0x80,       // Char 9b, Char 9a
    0x00, 0x80,       // Char 10b, Char 10a
    0x00, 0x80,       // Char 11b, Char 11a
    0x00, 0x80,       // Char 12b, Char 12a
    0x00, 0x80,       // Char 13b, Char 13a
    0x00, 0x80        // Char 14b, Char 14a
};

/***************************************************************************//*!
*
* @brief    Write one character to LCD   
*
* @param    character (ASCII format), support only '0'..'9',':','a'..'z'
*           position of LCD digit (0..13)
* @return   nothing
*
******************************************************************************/
void vfnLCD_Write_Char (unsigned char lbValue, unsigned char pos)
{
    unsigned char *lbpLCDWF;
    //unsigned char lbCounter;
    unsigned char lbShadow;
    unsigned char lbIndex;

    /* Only ASCII character, numbers and space */
    if (lbValue >='a' && lbValue <='z') lbValue -= 32;  // UpperCase
    if (lbValue >='A' && lbValue <='Z') lbValue -= 6;   // Only characters and numbers
    if (lbValue < '0' || lbValue > 'Z') lbValue = ':';  // default value as space

    lbValue -= '0';       // Remove the offset to search in the ASCII table
    lbValue *= _LCDTYPE;  // Compensate matrix offset

    lbIndex = (unsigned char)(1 + ((pos&0x0f) * _LCDTYPE));
    lbpLCDWF = bpaPlace[lbIndex];
    lbShadow  = *lbpLCDWF;
    lbShadow &= SEG_MASK;
    lbShadow |= baAscii[lbValue  + 1];
    *lbpLCDWF = lbShadow;

    lbIndex = (unsigned char)((pos&0x0f) * _LCDTYPE);
    lbpLCDWF = bpaPlace[lbIndex];
    *lbpLCDWF = baAscii[lbValue]; 
}

/***************************************************************************//*!
*
* @brief    Write message to upper or lower LCD digits  
*
* @param    message (ASCII format)
* @return   nothing
*
******************************************************************************/
void  vfnLCD_Write_Msg (unsigned char maxDigits, unsigned char *lbpMessage)
{
    unsigned char lbSize = 0;          
    while (lbSize < maxDigits  && *lbpMessage) 
    {
       vfnLCD_Write_Char (*lbpMessage,bLCD_CharPosition++);
       lbSize++;    
       lbpMessage++;
    }
}
 
/***************************************************************************//*!
*
* @brief    Text message roll  
*
* @param    text for scrolling on upper LCD digits
* @return   nothing
*
******************************************************************************/
  void vfnLCD_MessageRoll (unsigned char *text) {
    unsigned char i,j;
    unsigned char *pTextAll,*pTextScr;

    pTextAll = text;                   // initial text message on LCD
    for (i=0;i<strlen((const char*)text);i++) {
      pTextScr = pTextAll;
      for (j=0;j<_LCD_CHARNUM1;j++) {
        LCD_buffer[j] = *pTextScr++;
      }
      LCD_buffer[_LCD_CHARNUM1] = '\0';
      vfnLCD_All_Segments_OFF();
      bLCD_CharPosition = 1;  //Home display
      vfnLCD_Write_Msg(_LCD_CHARNUM1,LCD_buffer);
      arch_delay(20000000l);          // wait ms         
      pTextAll++;
    }
  }

/***************************************************************************//*!
*
* @brief    Write string to LCD 
*
* @param    string (if > 8 digits then string is scrolled) - upper stage
*           string (if > 5 digits then string is cut-off) - lower stage
*           stage=1 for upper stage, stage=2 for lower stage
* @return   nothing
* @note     when number and text is written on one LCD stage, it is recommended
*           to call vfnLCD_Write_Num firstly and then vfnLCD_Write_Str, because
*           vfnLCD_Write_Num fills up all LCD digits on the stage
*           String is lined up from left to right
******************************************************************************/
void  vfnLCD_Write_Str (unsigned char *lbpMessage, unsigned char stage)
{
    unsigned char lbSize = 0;
    unsigned char *pMsg;

    if ((stage == 1) || (stage == 2)) {       // only stage 1 or 2
        pMsg = lbpMessage;
        while(*pMsg++) lbSize++;
        
        if (stage == 1) {
          bLCD_CharPosition=1;                //upper part of display
         }
        else
        {
          if (lbSize > (_MAX_NUM_DIGITS2) ) 
            lbSize = _MAX_NUM_DIGITS2;        // cut-off remaining digits on lower stage
          bLCD_CharPosition=9;                //lower part of display
        }
        
        if (lbSize > (_MAX_NUM_DIGITS1) )
        {
          // string on upper LCD stage is too long to show - must be scrolled
          vfnLCD_MessageRoll(lbpMessage);   
        }
        else
        {
           do
            {
              vfnLCD_Write_Char (*lbpMessage,bLCD_CharPosition++);
              lbpMessage ++;     
            } while (--lbSize);
        }
    }
}
 
/***************************************************************************//*!
*
* @brief    Turn-on all LCD segments 
*
* @param    nothing
* @return   nothing
*
******************************************************************************/
void vfnLCD_All_Segments_ON(void)
{              
/*
    unsigned char lbCounter = 40;     // max. 31 frontplane pins
    unsigned char *lbpLCDWF;
    
    while (lbCounter--)
    {
       lbpLCDWF  = bpaPlace[19] + lbCounter;  // Char10b is starting pin !
       *lbpLCDWF = ALLON; 
    }
*/
    LCD_WF7TO4   = 0x0F0F0F0F;  
    LCD_WF11TO8  = 0x0F0F0F0F;  
    LCD_WF15TO12 = 0x0F0F0F0F;  
    LCD_WF19TO16 = 0x0F0F0F0F;  
    LCD_WF23TO20 = 0x00000F0F;  
    LCD_WF27TO24 = 0x00000F0F;  
    LCD_WF31TO28 = 0x0F0F0F0F;  
    LCD_WF35TO32 = 0x0F0F0F0F;  
    LCD_WF39TO36 = 0x0F0F0F0F;  
    LCD_WF43TO40 = 0x0F0F0F0F;  
    LCD_WF47TO44 = 0x0F0F0F0F;  
}

/***************************************************************************//*!
*
* @brief    Turn-off all LCD segments 
*
* @param    nothing
* @return   nothing
*
******************************************************************************/
void vfnLCD_All_Segments_OFF (void)
{              
/*
    unsigned char lbCounter = 40;     // max. 31 frontplane pins
    unsigned char *lbpLCDWF;

    while (lbCounter --)
    {
      lbpLCDWF  = bpaPlace[19] + lbCounter; // Char10b is starting pin !    
      *lbpLCDWF = ALLOFF;
    }
*/
 
    LCD_WF7TO4   = 0x00000000;  
    LCD_WF11TO8  = 0x00000000;  
    LCD_WF15TO12 = 0x00000000;  
    LCD_WF19TO16 = 0x00000000;  
    LCD_WF23TO20 = 0x00000000;  
    LCD_WF27TO24 = 0x00000000;  
    LCD_WF31TO28 = 0x00000000;  
    LCD_WF35TO32 = 0x00000000;  
    LCD_WF39TO36 = 0x00000000;  
    LCD_WF43TO40 = 0x00000000;  
    LCD_WF47TO44 = 0x00000000;  
}

/***************************************************************************//*!
*
* @brief    Fill LCD buffer with number and send it to LCD 
*
* @param    number - max. 7 digits with sign or 8 digits without sign
*           mindigits - minimum number of digits to show on LCD (number format)
*           stage - 1=upper LCD stage (9 digits max.), 2=lower LCD stage (5 digits max.)
* @return   nothing
* @note     when number and text is written on one LCD stage, it is recommended
*           to call vfnLCD_Write_Num firstly and then vfnLCD_Write_Str, because
*           vfnLCD_Write_Num fills up all LCD digits on the stage
*           Number is lined up from right to left
******************************************************************************/
void vfnLCD_Write_Num(long number, unsigned char minDigits, unsigned char stage) {
/*
  LCD_WF43       = baAscii[0];
  LCD_WF42       = baAscii[1];

  LCD_WF30       = baAscii[2];
  LCD_WF29       = baAscii[3];

  LCD_WF24       = baAscii[4];
  LCD_WF21       = baAscii[5];
  
  LCD_WF20       = baAscii[6];
  LCD_WF19       = baAscii[7];
  
  LCD_WF18       = baAscii[8];
  LCD_WF17       = baAscii[9];

  LCD_WF16       = baAscii[10];
  LCD_WF15       = baAscii[11];

  LCD_WF14       = baAscii[12];
  LCD_WF13       = baAscii[13];

  LCD_WF10       = baAscii[14];
  LCD_WF9        = baAscii[15];

  LCD_WF8        = baAscii[16];
  LCD_WF7        = baAscii[17];
  
  LCD_WF6        = baAscii[18];
  LCD_WF5        = baAscii[19];

  LCD_WF40       = baAscii[22];
  LCD_WF39       = baAscii[23];

  LCD_WF38       = baAscii[24];
  LCD_WF37       = baAscii[25];

  LCD_WF36       = baAscii[26];
  LCD_WF35       = baAscii[27];

  LCD_WF34       = baAscii[28];
  LCD_WF33       = baAscii[29];

  LCD_WF32       = baAscii[30];
  LCD_WF31       = baAscii[31];
*/  
  
  unsigned char dig,i;
  long reghelp;
  unsigned char position=0;
  long decimal;
  unsigned char numPos=0;
  unsigned char tempLCDb[_MAX_NUM_DIGITS1];
  unsigned char *s=LCD_buffer;
  unsigned char maxDigits;
  
  if ((stage == _LINE1_UPPER)||(stage == _LINE2_LOWER)||(stage == _LINE3_ALONE))  // only stage 1, 2 or 3
  {
      if (number < 0)
        reghelp=-number;
      else 
        reghelp=number;
      
      if (stage == _LINE1_UPPER ) 
      {
        maxDigits = _MAX_NUM_DIGITS1;                       // number of digits overflow
        bLCD_CharPosition = _LINE1_START;                   // upper part of display start number
        decimal=100000000;
       }
      else if (stage == _LINE2_LOWER )
      {
        maxDigits = _MAX_NUM_DIGITS2;                       // number of digits overflow
        bLCD_CharPosition = _LINE2_START;                   // lower part of display start number
        decimal=10000;
      }
      else
      {
        maxDigits = _MAX_NUM_DIGITS3;                       // number of digits overflow
        bLCD_CharPosition = _LINE3_START;                   // stand alone part of display start number
        decimal=1;
      }
        
      
      if (minDigits>maxDigits) minDigits = maxDigits;       // number of digits overflow
     
      for (i=0;i<maxDigits;i++){
        dig=(unsigned char)(reghelp / decimal);                    
        if ((dig==0) && (!numPos)) {
          tempLCDb[position++] = (unsigned char) ('0');     // fill zero to upper left digits
        }
        else {
          tempLCDb[position++] = (unsigned char) (dig + '0');// fill numbers
          numPos++;
        }
        reghelp= (long)(reghelp-dig*decimal);             
        decimal /= 10;
      }
      if (numPos<minDigits) numPos = minDigits;             // number of digits is less than minimum represented digits
      
      position = 0;
    
      for (i=0;i<(maxDigits-numPos);i++){                   // normal state of digits
        s[position++] = ' ';
      }
      i = (unsigned char)(maxDigits-numPos);
    
      for (;i<maxDigits;i++){                               // fill number
        s[position++] = tempLCDb[i];
      }
      
      s[position] = '\0';                                   // end of string
    
      vfnLCD_Write_Msg(maxDigits,LCD_buffer);               // send data to LCD !
      /*
      if ((stage == 1) && (LCD_buffer[0] == ' ')) 
      { 
        if (number<0) _LCD_MINUS_ON()
        else _LCD_MINUS_OFF();
      }
      */
  }
}

/***************************************************************************//*!
*
* @brief    Turns-on/off special symbol on the LCD
*
* @param    pin specified LCD_WFyTOx register
*           com specified position in register LCD_WFyTOx
*           mode 0=OFF / 1=ON / 2=TOGGLE
* @return   nothing
* @note     
*
******************************************************************************/
void vfnLCD_Edit_Symbol (unsigned char pin, unsigned char com, unsigned char mode)
{
    unsigned char *lbpLCDWF;
    unsigned char lbShadow;

    lbpLCDWF  = ((unsigned char *)&LCD_WF3TO0 + pin);       //Initialize pointer base address
    lbShadow  = *lbpLCDWF;                                  //Load Actual value
    
    switch (mode)                                           //Action according "mode"
    {
      case SYMBOL_OFF:
        lbShadow &= (~com);                                 //Clear by logical AND  
        break;  

      case SYMBOL_ON:
        lbShadow |= com;                                    //Set by logical OR   
        break;  

      case SYMBOL_TOGGLE:
        lbShadow ^= com;                                    //Toggle by logical XOR
        break;  
    }
      
    *lbpLCDWF = lbShadow;                                   //Store change value
}