/******************************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      LCD.h
* @author    b19186
* @version   0.0.1.0
* @date      April-11-2011
* @brief     Driver header for LCD, type MakSay 150020254
*
******************************************************************************/
#ifndef __LCD_H
#define __LCD_H

/******************************************************************************/
/**** General purpose functions constant definition ***************************/
/******************************************************************************/
  /****************************************************************************/
  /**** Constant for function Edit_symbol *************************************/
  /****************************************************************************/
#define SYMBOL_OFF    0x00
#define SYMBOL_ON     0x01
#define SYMBOL_TOGGLE 0x02

/******************************************************************************/
/**** General purpose functions ***********************************************/
/******************************************************************************/

    /* Initialization of the LCD */
    void vfnLCD_Init  (void);
    
    /* Change of contrast of the LCD */
    void vfnLCD_Contrast (unsigned char lbContrast);
    
    /* Show one ASCII character on the LCD on demand position (0..13) */
    void vfnLCD_Write_Char (unsigned char lbValue, unsigned char pos);
   
    /* Write string to LCD - writting to upper (1) or lower (2) stage, when string
    is too long then it is scrolled (upper) or cut-off (lower) */
    void  vfnLCD_Write_Str (unsigned char *lbpMessage, unsigned char stage);
    
    /* Write number to LCD (number with sign for upper 8 digits, without sign for lower 5 digits,
    minDigits=minimum number of digits to show, writting to upper (1) or lower (2) stage */
    void vfnLCD_Write_Num (long number, unsigned char minDigits, unsigned char stage);
    
    /* Turns-on/off special symbol on the LCD, pin specified LCD_WFyTOx register, 
    com specified position in register LCD_WFyTOx, mode 0 - OFF / 1 - ON  */
    void vfnLCD_Edit_Symbol (unsigned char pin, unsigned char com, unsigned char mode);

    /* Turns-on all the segments on the LCD */
    void vfnLCD_All_Segments_ON (void);
                                                                                       
    /* Turns-off all the segments on the LCD */
    void vfnLCD_All_Segments_OFF (void);

/********************** General purpose functions *****************************/

#endif /* __LCD_H */