/******************************************************************************
* 
* Copyright (c) 2008 Freescale Semiconductor;
* All Rights Reserved                       
*
*******************************************************************************
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
***************************************************************************//*!
*
* @file      LCD_glass.h
* @author    b02785
* @version   0.0.1.0
* @date      09-01-2011
* @brief     Header for LCD glas for 3-phase energy meter
*
******************************************************************************/
#ifndef __LCD_GLASS_H
#define __LCD_GLASS_H

//******************************************************************************
//**** LCD common constant *****************************************************
//******************************************************************************
  //****************************************************************************
  //**** Number of digits on the line ******************************************
  //****************************************************************************
    #define _LCD_CHARNUM1   (9)                       //Max. number of upper LCD segments
    #define _LCD_CHARNUM2   (5)                       //Max. number of lower LCD segments
    #define _LCD_CHARNUM3   (1)                       //Max. number of stand alone LCD segments
    #define _MAX_NUM_DIGITS1  (_LCD_CHARNUM1)         //Max. number of LCD digits for upper segments 
    #define _MAX_NUM_DIGITS2  (_LCD_CHARNUM2)         //Max. number of LCD digits for lower segments 
    #define _MAX_NUM_DIGITS3  (_LCD_CHARNUM3)         //Max. number of LCD digits for stand alone segments
    #define _LINE1_UPPER    0x01                      //The first char on the upper line 
    #define _LINE2_LOWER    0x02                      //The first char on the lower line 
    #define _LINE3_ALONE    0x03                      //The first char on the stand alone line 
    #define _LINE1_START    0x00                      //The first char on the upper line 
    #define _LINE2_START    0x09                      //The first char on the lower line 
    #define _LINE3_START    0x0E                      //The first char on the stand alone line 
  //****************************************************************************
  //**** Number of write for one 7 segment digit *******************************
  //****************************************************************************
    #define _LCDTYPE        (2)                       //4 backplane => 2 writes for 7bits
  //****************************************************************************
  //**** Constant for "Lamp Test" **********************************************
  //****************************************************************************
    #define  ALLON          0x0F                      //All segments ON
    #define  ALLOFF         0x00                      //All segments OFF  
  //****************************************************************************
  //**** Position of digits on the lines ***************************************
  //****************************************************************************
    #define _LINE1_P00      0x00                      //Line 1 position MsD
    #define _LINE1_P01      0x01                      //Line 1 position
    #define _LINE1_P02      0x02                      //Line 1 position
    #define _LINE1_P03      0x03                      //Line 1 position
    #define _LINE1_P04      0x04                      //Line 1 position
    #define _LINE1_P05      0x05                      //Line 1 position
    #define _LINE1_P06      0x06                      //Line 1 position
    #define _LINE1_P07      0x07                      //Line 1 position
    #define _LINE1_P08      0x08                      //Line 1 position LsD
    #define _LINE2_P00      0x09                      //Line 2 position MsD
    #define _LINE2_P01      0x0A                      //Line 2 position
    #define _LINE2_P02      0x0B                      //Line 2 position
    #define _LINE2_P03      0x0C                      //Line 2 position
    #define _LINE2_P04      0x0D                      //Line 2 position
    #define _LINE3_P00      0x0E                      //Line 3 position MsD

//******************************************************************************
//**** Backplane constant ******************************************************
//******************************************************************************
  //****************************************************************************
  //**** "COMx" definition for symbols ON/OFF macros ***************************
  //****************************************************************************
    #define   COM1          0x08                      //Definition of the "COM1"
    #define   COM2          0x04                      //Definition of the "COM2"
    #define   COM3          0x02                      //Definition of the "COM3"
    #define   COM4          0x01                      //Definition of the "COM4"
    //**************************************************************************
    //**** Back plane initialize value for LCD_WF3TOx **************************
    //**************************************************************************
    #define   WF3TOx_BACKPL ((COM4 << 0)|(COM3 << 8)|(COM2 << 16)|(COM1 << 24))

//******************************************************************************
//**** Indicate witch segments of Character are connected to MCU LCDn pins *****
//******************************************************************************
//**** Each segment must be connected to the two MCU pins                   ****
//**** Seg1_X are segmants: A, D, E, F                                      ****  
//**** Seg2_X are segmants: B, C, G                                         ****  
//******************************************************************************
    #define   SegmentPlace(LCDn) *((unsigned char *)&LCD_WF3TO0 + LCDn)
  //****************************************************************************
  //**** Main-upper Segment Row ************************************************
  //****************************************************************************
    #define   Seg1_01       SegmentPlace (30)         // LCD Pin 18, MCU LCD30
    #define   Seg2_01       SegmentPlace (29)         // LCD Pin 19, MCU LCD29
    #define   Seg1_02       SegmentPlace (24)         // LCD Pin 22, MCU LCD24
    #define   Seg2_02       SegmentPlace (21)         // LCD Pin 23, MCU LCD21
    #define   Seg1_03       SegmentPlace (20)         // LCD Pin 24, MCU LCD20
    #define   Seg2_03       SegmentPlace (19)         // LCD Pin 25, MCU LCD19
    #define   Seg1_04       SegmentPlace (18)         // LCD Pin 26, MCU LCD18
    #define   Seg2_04       SegmentPlace (17)         // LCD Pin 27, MCU LCD17
    #define   Seg1_05       SegmentPlace (16)         // LCD Pin 28, MCU LCD16
    #define   Seg2_05       SegmentPlace (15)         // LCD Pin 29, MCU LCD15
    #define   Seg1_06       SegmentPlace (14)         // LCD Pin 30, MCU LCD14
    #define   Seg2_06       SegmentPlace (13)         // LCD Pin 31, MCU LCD13
    #define   Seg1_07       SegmentPlace (10)         // LCD Pin 34, MCU LCD10
    #define   Seg2_07       SegmentPlace (9)          // LCD Pin 35, MCU LCD9
    #define   Seg1_08       SegmentPlace (8)          // LCD Pin 36, MCU LCD8
    #define   Seg2_08       SegmentPlace (7)          // LCD Pin 37, MCU LCD7
    #define   Seg1_09       SegmentPlace (6)          // LCD Pin 38, MCU LCD6
    #define   Seg2_09       SegmentPlace (5)          // LCD Pin 39, MCU LCD5
  //****************************************************************************
  //**** Auxiliary-lower Segment Row *******************************************
  //****************************************************************************
    #define   Seg1_10       SegmentPlace (40)         // LCD Pin 8,  MCU LCD40
    #define   Seg2_10       SegmentPlace (39)         // LCD Pin 9,  MCU LCD39
    #define   Seg1_11       SegmentPlace (38)         // LCD Pin 10, MCU LCD38
    #define   Seg2_11       SegmentPlace (37)         // LCD Pin 11, MCU LCD37
    #define   Seg1_12       SegmentPlace (36)         // LCD Pin 12, MCU LCD36
    #define   Seg2_12       SegmentPlace (35)         // LCD Pin 13, MCU LCD35
    #define   Seg1_13       SegmentPlace (34)         // LCD Pin 14, MCU LCD34
    #define   Seg2_13       SegmentPlace (33)         // LCD Pin 15, MCU LCD33
    #define   Seg1_14       SegmentPlace (32)         // LCD Pin 16, MCU LCD32
    #define   Seg2_14       SegmentPlace (31)         // LCD Pin 17, MCU LCD31
  //****************************************************************************
  //**** Stand alone Segment ***************************************************
  //****************************************************************************
    #define   Seg1_15       SegmentPlace (43)         // LCD Pin 5,  MCU LCD42
    #define   Seg2_15       SegmentPlace (42)         // LCD Pin 6,  MCU LCD43

//******************************************************************************
//**** Indicate witch Part of Symbols are connected to MCU LCDn pins ***********
//******************************************************************************
    //#define   SymbolON(LCDn, mask)    {*((unsigned char *)&LCD_WF3TO0 + LCDn) |= (1<<(mask));} 
    //#define   SymbolOFF(LCDn, mask)   {*((unsigned char *)&LCD_WF3TO0 + LCDn) &= ~(1<<(mask));}  
  //****************************************************************************
  //**** Symbols Pin definition ************************************************
  //****************************************************************************
    #define   _LCD_BATT1_PIN    46
    #define   _LCD_BATT2_PIN    28
    #define   _LCD_RS_COM_PIN   10
    #define   _LCD_LQ_PIN       20
    #define   _LCD_CQ_PIN       20
    #define   _LCD_I1_PIN       47
    #define   _LCD_I1_L_PIN     28
    #define   _LCD_I1_R_PIN     25
    #define   _LCD_I2_PIN       4
    #define   _LCD_I2_L_PIN     25
    #define   _LCD_I2_R_PIN     25
    #define   _LCD_I3_PIN       11
    #define   _LCD_I3_L_PIN     25
    #define   _LCD_I3_R_PIN     12
    #define   _LCD_L1_PIN       12
    #define   _LCD_L2_PIN       12    
    #define   _LCD_L3_PIN       12    
    #define   _LCD_TRG_PIN      4                       //Triangle
    #define   _LCD_LOCK_PIN     4    
    #define   _LCD_ANTEN_PIN    11
    #define   _LCD_CLOCK_PIN    47    
    #define   _LCD_COSFI_PIN    41    
    #define   _LCD_SM_I_PIN     41                      //Small "i"
    #define   _LCD_SM_C_PIN     41                      //Small "c"
    #define   _LCD_RMS_PIN      42         
    #define   _LCD_STAR_PIN     44                      //Small Star
    #define   _LCD_P1_PIN       19         
    #define   _LCD_P2_PIN       15
    #define   _LCD_P3_PIN       13
    #define   _LCD_P4_PIN       39
    #define   _LCD_P5_PIN       37
    #define   _LCD_P6_PIN       35
    #define   _LCD_P7_PIN       33
    #define   _LCD_T1_PIN       31
    #define   _LCD_T2_PIN       29
    #define   _LCD_T3_PIN       21
    #define   _LCD_T4_PIN       17
    #define   _LCD_HZ_PIN       11
    #define   _LCD_BG_K_PIN     41
    #define   _LCD_BG_V_PIN     9
    #define   _LCD_ARH_PIN      7
    #define   _LCD_KW_PIN       5
    #define   _LCD_BG_H_PIN     4

    #define   _LCD_VIR_S3_PIN   47
    #define   _LCD_VIR_S4_PIN   46
    #define   _LCD_VIR_S5_PIN   45
    #define   _LCD_VIR_S6_PIN   44
    #define   _LCD_VIR_S7_PIN   47
    #define   _LCD_VIR_S8_PIN   46
    #define   _LCD_VIR_S9_PIN   45
    #define   _LCD_VIR_S10_PIN  44
    #define   _LCD_VIR_S11_PIN  45
    #define   _LCD_VIR_S12_PIN  46
    #define   _LCD_VIR_S13_PIN  45
    #define   _LCD_VIR_S14_PIN  44


  //****************************************************************************
  //**** Symbols COM definition ************************************************
  //****************************************************************************
    #define   _LCD_BATT1_COM    COM1    
    #define   _LCD_BATT2_COM    COM1    
    #define   _LCD_RS_COM_COM   COM2    
    #define   _LCD_LQ_COM       COM2    
    #define   _LCD_CQ_COM       COM3    
    #define   _LCD_I1_COM       COM4    
    #define   _LCD_I1_L_COM     COM4    
    #define   _LCD_I1_R_COM     COM2    
    #define   _LCD_I2_COM       COM4    
    #define   _LCD_I2_L_COM     COM3    
    #define   _LCD_I2_R_COM     COM4    
    #define   _LCD_I3_COM       COM3    
    #define   _LCD_I3_L_COM     COM1    
    #define   _LCD_I3_R_COM     COM4    
    #define   _LCD_L1_COM       COM3    
    #define   _LCD_L2_COM       COM2    
    #define   _LCD_L3_COM       COM1    
    #define   _LCD_TRG_COM      COM2                    //Triangle
    #define   _LCD_LOCK_COM     COM3    
    #define   _LCD_ANTEN_COM    COM4    
    #define   _LCD_CLOCK_COM    COM3    
    #define   _LCD_COSFI_COM    COM4    
    #define   _LCD_SM_I_COM     COM3                    //Small "i"
    #define   _LCD_SM_C_COM     COM2                    //Small "c"
    #define   _LCD_RMS_COM      COM1         
    #define   _LCD_STAR_COM     COM1                   //Small Star
    #define   _LCD_P1_COM       COM1         
    #define   _LCD_P2_COM       COM1
    #define   _LCD_P3_COM       COM1
    #define   _LCD_P4_COM       COM1
    #define   _LCD_P5_COM       COM1
    #define   _LCD_P6_COM       COM1
    #define   _LCD_P7_COM       COM1
    #define   _LCD_T1_COM       COM1
    #define   _LCD_T2_COM       COM1
    #define   _LCD_T3_COM       COM1
    #define   _LCD_T4_COM       COM1
    #define   _LCD_HZ_COM       COM1
    #define   _LCD_BG_K_COM     COM1
    #define   _LCD_BG_V_COM     COM1
    #define   _LCD_ARH_COM      COM1
    #define   _LCD_KW_COM       COM1
    #define   _LCD_BG_H_COM     COM1
//    #define   _LCD_KVARH    SymbolON(41,COM1)
    #define   _LCD_VIR_S3_COM   COM1
    #define   _LCD_VIR_S4_COM   COM2
    #define   _LCD_VIR_S5_COM   COM1
    #define   _LCD_VIR_S6_COM   COM2
    #define   _LCD_VIR_S7_COM   COM2
    #define   _LCD_VIR_S8_COM   COM3
    #define   _LCD_VIR_S9_COM   COM2
    #define   _LCD_VIR_S10_COM  COM3
    #define   _LCD_VIR_S11_COM  COM3
    #define   _LCD_VIR_S12_COM  COM4
    #define   _LCD_VIR_S13_COM  COM4
    #define   _LCD_VIR_S14_COM  COM4

  //****************************************************************************
  //**** Symbols COM definition ************************************************
  //****************************************************************************
/*
    #define   _LCD_BATT1_COM  COM1    
    #define   _LCD_BATT2_COM  COM1    
    #define   _LCD_RS_COM_COM COM2    
    #define   _LCD_LQ_COM     COM2    
    #define   _LCD_CQ_COM     COM3    
    #define   _LCD_I1       SymbolON(47,COM4)    
    #define   _LCD_I1_L     SymbolON(28,COM4)    
    #define   _LCD_I1_R     SymbolON(25,COM2)    
    #define   _LCD_I2       SymbolON(4,COM4)    
    #define   _LCD_I2_L     SymbolON(25,COM3)    
    #define   _LCD_I2_R     SymbolON(25,COM4)    
    #define   _LCD_I3       SymbolON(11,COM3)    
    #define   _LCD_I3_L     SymbolON(25,COM4)    
    #define   _LCD_I3_R     SymbolON(12,COM4)    
    #define   _LCD_L1       SymbolON(11,COM3)    
    #define   _LCD_L2       SymbolON(11,COM2)    
    #define   _LCD_L3       SymbolON(11,COM1)    
    #define   _LCD_TRG      SymbolON(1,COM2)          //Triangle
    #define   _LCD_LOCK     SymbolON(1,COM3)    
    #define   _LCD_ANTEN    SymbolON(11,COM4)    
    #define   _LCD_CLOCK    SymbolON(47,COM4)    
    #define   _LCD_COSFI    SymbolON(41,COM4)    
    #define   _LCD_SM_I     SymbolON(41,COM3)         //Small "i"
    #define   _LCD_SM_C     SymbolON(41,COM2)         //Small "c"
    #define   _LCD_RMS      SymbolON(42,COM1)         
    #define   _LCD_STAR     SymbolON(44,COM1)         //Small Star
    #define   _LCD_P1       SymbolON(19,COM1)         
    #define   _LCD_P2       SymbolON(15,COM1)
    #define   _LCD_P3       SymbolON(13,COM1)
    #define   _LCD_P4       SymbolON(39,COM1)
    #define   _LCD_P5       SymbolON(37,COM1)
    #define   _LCD_P6       SymbolON(35,COM1)
    #define   _LCD_P7       SymbolON(33,COM1)
    #define   _LCD_T1       SymbolON(31,COM1)
    #define   _LCD_T2       SymbolON(29,COM1)
    #define   _LCD_T3       SymbolON(21,COM1)
    #define   _LCD_T4       SymbolON(17,COM1)
    #define   _LCD_HZ       SymbolON(11,COM1)
    #define   _LCD_BG_K     SymbolON(41,COM1)
    #define   _LCD_BG_V     SymbolON(9,COM1)
    #define   _LCD_ARH      SymbolON(7,COM1)
    #define   _LCD_KW       SymbolON(5,COM1)
    #define   _LCD_BG_H     SymbolON(4,COM1)
//    #define   _LCD_KVARH    SymbolON(41,COM1)
*/
//******************************************************************************
//**** Segment constant ********************************************************
//******************************************************************************
  //****************************************************************************
  //**** Seg1_X A, D, E, F position in the WFx field in LCD_WF3TOx *************
  //****************************************************************************
    #define  SEGA           0x01                      //Position of Segment "A"
    #define  SEGF           0x02                      //Position of Segment "F"
    #define  SEGE           0x04                      //Position of Segment "E"
    #define  SEGD           0x08                      //Position of Segment "D"
  //****************************************************************************
  //**** Seg2_X B, C, G position in the WFx field in LCD_WF3TOx ****************
  //****************************************************************************
    #define  SEGB           0x01                      //Position of Segment "B"
    #define  SEGG           0x02                      //Position of Segment "G"
    #define  SEGC           0x04                      //Position of Segment "C"
    #define  SEG_MASK       0x80                      //Position for masking
  //****************************************************************************
  //**** 7 segment code mask ***************************************************
  //****************************************************************************
    #define  N0_1          (( SEGA)|( SEGD)|( SEGE)|( SEGF)) 
    #define  N0_2          (( SEGB)|( SEGC)|(!SEGG))  //Char = 0,   offset=0
    #define  N1_1          ((!SEGA)|(!SEGD)|(!SEGE)|(!SEGF)) 
    #define  N1_2          (( SEGB)|( SEGC)|(!SEGG))  //Char = 1,   offset=2
    #define  N2_1          (( SEGA)|( SEGD)|( SEGE)|(!SEGF)) 
    #define  N2_2          (( SEGB)|(!SEGC)|( SEGG))  //Char = 2,   offset=4
    #define  N3_1          (( SEGA)|( SEGD)|(!SEGE)|(!SEGF)) 
    #define  N3_2          (( SEGB)|( SEGC)|( SEGG))  //Char = 3,   offset=6
    #define  N4_1          ((!SEGA)|(!SEGD)|(!SEGE)|( SEGF)) 
    #define  N4_2          (( SEGB)|( SEGC)|( SEGG))  //Char = 4,   offset=8
    #define  N5_1          (( SEGA)|( SEGD)|(!SEGE)|( SEGF)) 
    #define  N5_2          ((!SEGB)|( SEGC)|( SEGG))  //Char = 5,   offset=10
    #define  N6_1          (( SEGA)|( SEGD)|( SEGE)|( SEGF)) 
    #define  N6_2          ((!SEGB)|( SEGC)|( SEGG))  //Char = 6,   offset=12
    #define  N7_1          (( SEGA)|(!SEGD)|(!SEGE)|(!SEGF)) 
    #define  N7_2          (( SEGB)|( SEGC)|(!SEGG))  //Char = 7,   offset=14
    #define  N8_1          (( SEGA)|( SEGD)|( SEGE)|( SEGF)) 
    #define  N8_2          (( SEGB)|( SEGC)|( SEGG))  //Char = 8,   offset=16
    #define  N9_1          (( SEGA)|( SEGD)|(!SEGE)|( SEGF)) 
    #define  N9_2          (( SEGB)|( SEGC)|( SEGG))  //Char = 9,   offset=18
    #define  SP_1          ((!SEGA)|(!SEGD)|(!SEGE)|(!SEGF)) 
    #define  SP_2          ((!SEGB)|(!SEGC)|(!SEGG))  //Char =  ,   offset=20
    #define  CA_1          (( SEGA)|(!SEGD)|( SEGE)|( SEGF)) 
    #define  CA_2          (( SEGB)|( SEGC)|( SEGG))  //Char = A,   offset=22
    #define  CB_1          ((!SEGA)|( SEGD)|( SEGE)|( SEGF)) 
    #define  CB_2          ((!SEGB)|( SEGC)|( SEGG))  //Char = b,   offset=24
    #define  CC_1          ((!SEGA)|( SEGD)|( SEGE)|(!SEGF)) 
    #define  CC_2          ((!SEGB)|(!SEGC)|( SEGG))  //Char = c,   offset=26
    #define  CD_1          ((!SEGA)|( SEGD)|( SEGE)|(!SEGF)) 
    #define  CD_2          (( SEGB)|( SEGC)|( SEGG))  //Char = d,   offset=28
    #define  CE_1          (( SEGA)|( SEGD)|( SEGE)|( SEGF)) 
    #define  CE_2          ((!SEGB)|(!SEGC)|( SEGG))  //Char = E,   offset=30
    #define  CF_1          (( SEGA)|(!SEGD)|( SEGE)|( SEGF)) 
    #define  CF_2          ((!SEGB)|(!SEGC)|( SEGG))  //Char = F,   offset=32

#endif /* __LCD_GLASS_H */