/*****************************************************************************
 * (c) Copyright 2010, Freescale Semiconductor Inc.
 * ALL RIGHTS RESERVED.
 ***************************************************************************//*!
 * @file      pdb.c
 * @author    R55013
 * @version   1.0.4.0
 * @date      Mar-8-2011
 * @brief     PDB driver implementation.
 * @par       Driver example:
 * @include   adcpdb_test\adcpdb_test.c
 * @par       Interrupt vector definition(s):
 * @include   adcpdb_test\isr.h		
 ******************************************************************************/
#include "common.h"
#include "pdb.h"

/******************************************************************************
 * data type definitions                                                      *
 ******************************************************************************/
static PDB_CALLBACK volatile pCallbackPDB=NULL;

/******************************************************************************
 * interrupt functions prototype                                              *
 ******************************************************************************/
void PDB0_IRQHandler (void);

/******************************************************************************
 * Public functions definitions                                               *
 ******************************************************************************/
/***************************************************************************//*!
 * @brief   PDB module initialization function.
 * @param   pdb      - PDB configuration structure passed by value: 
 *                     PDB_MODULE_SWTRG_CONFIG1.
 * @param   ip       - interrupt priority 0..15
 * @param   pCallback- pointer to PDB_CALLBACK function
 * @return  none
 * @note    Implemented as function call.
 ******************************************************************************/
#pragma diag_suppress=Pa082
void PDB_Init (tPDB pdb, uint8 ip, PDB_CALLBACK pCallback)
{
  if (pCallback != NULL) 
  { 
    pCallbackPDB = pCallback; 
    NVIC_EnableIRQ(PDB0_IRQn); 
  }
  PDB0->SC    = (PDB_SC_LDMOD(0) | PDB_SC_PDBEN_MASK);
  PDB0->MOD   = pdb.MOD;
  PDB0->IDLY  = pdb.IDLY;
  PDB0->SC   |= PDB_SC_LDOK_MASK;
  PDB0->SC    = pdb.SC;
}

void PDBCH0TRG0_Init (tPDB_TRG trg, uint16 delay)
{
    PDB0->CH[0].DLY[0] = delay;
    PDB0->CH[0].C1  |= (((trg.C1&PDB_C1_BB_MASK )&(0x01<<PDB_C1_BB_SHIFT ))|
                   ((trg.C1&PDB_C1_TOS_MASK)&(0x01<<PDB_C1_TOS_SHIFT))|
                   ((trg.C1&PDB_C1_EN_MASK )&(0x01<<PDB_C1_EN_SHIFT)));  
}

void PDBCH0TRG1_Init (tPDB_TRG trg, uint16 delay)
{
   PDB0->CH[0].DLY[1] = delay;
   PDB0->CH[0].C1  |= (((trg.C1&PDB_C1_BB_MASK )&(0x02<<PDB_C1_BB_SHIFT ))|
                  ((trg.C1&PDB_C1_TOS_MASK)&(0x02<<PDB_C1_TOS_SHIFT))|
                  ((trg.C1&PDB_C1_EN_MASK )&(0x02<<PDB_C1_EN_SHIFT)));            
}

void PDBCH1TRG0_Init (tPDB_TRG trg, uint16 delay)
{
    PDB0->CH[1].DLY[0] = delay;
    PDB0->CH[1].C1  |= (((trg.C1&PDB_C1_BB_MASK )&(0x01<<PDB_C1_BB_SHIFT ))|
                   ((trg.C1&PDB_C1_TOS_MASK)&(0x01<<PDB_C1_TOS_SHIFT))|
                   ((trg.C1&PDB_C1_EN_MASK )&(0x01<<PDB_C1_EN_SHIFT)));            
}

void PDBCH1TRG1_Init (tPDB_TRG trg, uint16 delay)
{
    PDB0->CH[1].DLY[1] = delay;
    PDB0->CH[1].C1  |= (((trg.C1&PDB_C1_BB_MASK )&(0x02<<PDB_C1_BB_SHIFT ))|
                   ((trg.C1&PDB_C1_TOS_MASK)&(0x02<<PDB_C1_TOS_SHIFT))|
                   ((trg.C1&PDB_C1_EN_MASK )&(0x02<<PDB_C1_EN_SHIFT)));             
}

/******************************************************************************
 * interrupt functions definitions                                            *
 ******************************************************************************/
void PDB0_IRQHandler (void) 
{ 
  if (pCallbackPDB != (PDB_CALLBACK)NULL) 
  { 
    if ((PDB0->SC & PDB_SC_PDBIE_MASK) && (PDB0->SC & PDB_SC_PDBIF_MASK)) 
    { 
      PDB0->SC &= ~PDB_SC_PDBIF_MASK; 
      pCallbackPDB(PDBDLY_CALLBACK); 
    }
    
    if (PDB0->SC & PDB_SC_PDBEIE_MASK)  
    {
      if (PDB0->CH[0].S & PDB_S_ERR(3)) { PDB0->CH[0].S |= PDB_S_ERR(3); pCallbackPDB(CH0ERR_CALLBACK); }
      if (PDB0->CH[1].S & PDB_S_ERR(3)) { PDB0->CH[1].S |= PDB_S_ERR(3); pCallbackPDB(CH1ERR_CALLBACK); }
    }
  }
}
/******************************************************************************
 * End of module                                                              *
 ******************************************************************************/