/*****************************************************************************
 * (c) Copyright 2010, Freescale Semiconductor Inc.
 * ALL RIGHTS RESERVED.
 ***************************************************************************//*!
 * @file      pdb.h
 * @author    R55013
 * @version   1.0.4.0
 * @date      Mar-8-2011
 * @brief     PDB driver header file.
 ******************************************************************************/
#ifndef __PDB_H
#define __PDB_H 
 
/******************************************************************************
 * definition configuration structures                                        *
 ******************************************************************************/
typedef struct { uint32 SC, MOD, IDLY;  } tPDB;
typedef struct { uint32 C1; } tPDB_TRG;

/******************************************************************************
 * macro definitions                                                          *
 ******************************************************************************/
#define PDB_PRESCALER_1     (uint32)(0 << PDB_SC_PRESCALER_SHIFT)  
#define PDB_PRESCALER_2     (uint32)(1 << PDB_SC_PRESCALER_SHIFT)
#define PDB_PRESCALER_4     (uint32)(2 << PDB_SC_PRESCALER_SHIFT)
#define PDB_PRESCALER_8     (uint32)(3 << PDB_SC_PRESCALER_SHIFT)
#define PDB_PRESCALER_16    (uint32)(4 << PDB_SC_PRESCALER_SHIFT)
#define PDB_PRESCALER_32    (uint32)(5 << PDB_SC_PRESCALER_SHIFT)
#define PDB_PRESCALER_64    (uint32)(6 << PDB_SC_PRESCALER_SHIFT)
#define PDB_PRESCALER_128   (uint32)(7 << PDB_SC_PRESCALER_SHIFT)

#define PDB_MULT_1          (uint32)(0 << PDB_SC_MULT_SHIFT) 
#define PDB_MULT_10         (uint32)(1 << PDB_SC_MULT_SHIFT) 
#define PDB_MULT_20         (uint32)(2 << PDB_SC_MULT_SHIFT) 
#define PDB_MULT_40         (uint32)(3 << PDB_SC_MULT_SHIFT) 

/******************************************************************************
 * default configurations                                                     *
 ******************************************************************************/
#define PDB_MODULE_SWTRG_CONFIG1(presc,mult,modulo,idly)                      \
(tPDB){                                                                       \
/* SC       */  SET(PDB_SC_LDMOD(1))|CLR(PDB_SC_PDBEIE_MASK)|                 \
                SET(PDB_SC_SWTRIG_MASK)|CLR(PDB_SC_DMAEN_MASK)|presc|         \
                SET(PDB_SC_TRGSEL(15))|SET(PDB_SC_PDBEN_MASK)|                \
                SET(PDB_SC_PDBIE_MASK)|mult|SET(PDB_SC_CONT_MASK)|            \
                SET(PDB_SC_LDOK_MASK),                                        \
/* MOD      */  modulo,                                                       \
/* IDLY     */  idly,                                                         \
}

#define PDB_MODULE_SWTRG_NO_IRQ_CONFIG(presc,mult,modulo,idly)                \
(tPDB){                                                                       \
/* SC       */  SET(PDB_SC_LDMOD(0))|CLR(PDB_SC_PDBEIE_MASK)|                 \
                CLR(PDB_SC_SWTRIG_MASK)|CLR(PDB_SC_DMAEN_MASK)|presc|         \
                SET(PDB_SC_TRGSEL(15))|SET(PDB_SC_PDBEN_MASK)|                \
                CLR(PDB_SC_PDBIE_MASK)|mult|SET(PDB_SC_CONT_MASK)|            \
                CLR(PDB_SC_LDOK_MASK),                                        \
/* MOD      */  modulo,                                                       \
/* IDLY     */  idly,                                                         \
}

#define PDB_PRETRG_ON_DEALYED_CONFIG  /*                                   */ \
(tPDB_TRG){                                                                   \
/* C1 */CLR(PDB_C1_BB_MASK)|SET(PDB_C1_TOS_MASK)|SET(PDB_C1_EN_MASK)          \
}

#define PDB_PRETRG_ON_BYPASSED_CONFIG /*                                   */ \
(tPDB_TRG){                                                                   \
/* C1 */CLR(PDB_C1_BB_MASK)|CLR(PDB_C1_TOS_MASK)|SET(PDB_C1_EN_MASK)          \
}

#define PDB_PRETRG_OFF_CONFIG         /*                                   */ \
(tPDB_TRG){                                                                   \
/* C1 */CLR(PDB_C1_BB_MASK)|CLR(PDB_C1_TOS_MASK)|CLR(PDB_C1_EN_MASK)          \
}

/******************************************************************************
 * data type definitions                                                      *
 ******************************************************************************/
typedef enum { PDBDLY_CALLBACK=1, CH0ERR_CALLBACK=2, CH1ERR_CALLBACK=4 } PDB_CALLBACK_TYPE;
typedef void (*PDB_CALLBACK)(PDB_CALLBACK_TYPE type);

/* PDB macro definitions                                                      */
/***************************************************************************//*!
 * @brief   Macro enables PDB module.
 * @return  none
 * @note    Implemented as inlined macro.
 ******************************************************************************/
#define PDB_Enable()                    { PDB0->SC |=  PDB_SC_PDBEN_MASK; }

/***************************************************************************//*!
 * @brief   Macro disables PDB module.
 * @return  none
 * @note    Implemented as inlined macro.
 ******************************************************************************/
#define PDB_Disable()                   { PDB0->SC &= ~PDB_SC_PDBEN_MASK; }

/***************************************************************************//*!
 * @brief   Macro triggers PDB module execution.
 * @return  none
 * @note    Implemented as inlined macro.
 ******************************************************************************/
#define PDB_Trigger()                   { PDB0->SC |=  PDB_SC_SWTRIG_MASK; }

#define PDBCH0TRG0_SetDelay(delay)      { PDB[0]->CH[0]DLY[0] = delay;          }
#define PDBCH0TRG1_SetDelay(delay)      { PDB[0]->CH[0]DLY[1] = delay;          }
#define PDBCH1TRG0_SetDelay(delay)      { PDB[0]->CH[1]DLY[0] = delay;          }
#define PDBCH1TRG1_SetDelay(delay)      { PDB[0]->CH[1]DLY[1] = delay;          }
/***************************************************************************//*!
 * @brief   Macro sets trigger delay value.
 * @param   ch      - CH0|CH1
 * @param   trg     - TRG0|TRG1|TRG2|TRG3
 * @param   delay   - 16-bit delay value.
 * @note    Implemented as inlined macro.
 ******************************************************************************/
#define PDB_PTRG_SetDelay(ch,trg,delay) PDB##ch####trg##_SetDelay(delay)

/***************************************************************************//*!
 * @brief   Macro sets modulo register.
 * @param   modulo  - 16-bit modulo value
 * @note    Implemented as inlined macro.
 ******************************************************************************/
#define PDB_SetModulo(modulo)           { PDB0->MOD  = modulo;            }

/***************************************************************************//*!
 * @brief   Macro sets idelay register.
 * @param   delay  - 16-bit delay trigger value
 * @note    Implemented as inlined macro.
 ******************************************************************************/
#define PDB_SetIdelay(delay)            { PDB0->IDLY = delay;             }

/***************************************************************************//*!
 * @brief   Macro sets LDOK bit.
 * @note    Implemented as inlined macro.
 ******************************************************************************/
#define PDB_SetLDOK()                   { PDB0->SC|= PDB_SC_LDOK_MASK;    }

/***************************************************************************//*!
 * @brief   PDB trigger channel initialization function.
 * @param   ch      - CH0|CH1
 * @param   trg     - TRG0|TRG1|TRG2|TRG3
 * @param   cfg     - PDB trigger configuration structure passed by value: 
 *                    PDB_PRETRG_ON_DEALYED_CONFIG,
 *                    PDB_PRETRG_ON_BYPASSED_CONFIG,
 *                    PDB_PRETRG_OFF_CONFIG.
 * @param   delay   - 16-bit delay value 
 * @return  none
 * @note    Implemented as function call.
 ******************************************************************************/ 
#define PDB_PTRG_Init(ch,trg,cfg,delay) PDB##ch####trg##_Init(cfg,delay)
  
/******************************************************************************
 * public function prototypes                                                 *
 ******************************************************************************/    
extern void PDB_Init (tPDB pdb, uint8 ip, PDB_CALLBACK pCallback);
extern void PDBCH0TRG0_Init (tPDB_TRG trg, uint16 delay);
extern void PDBCH0TRG1_Init (tPDB_TRG trg, uint16 delay);
extern void PDBCH1TRG0_Init (tPDB_TRG trg, uint16 delay);
extern void PDBCH1TRG1_Init (tPDB_TRG trg, uint16 delay);
                               
#endif /* __PDB_H */ 