//******************************************************************************
//**** Sample project, ADC scam mode conversion with DMA support ***************
//******************************************************************************
//******************************************************************************
//**** File:	  Main.c   					            ****
//**** Version:	  0.0.1, 25.07.2012					    ****
//**** Company:   Freescale                                                 ****
//**** Author:	  B02785        					    ****
//******************************************************************************
#include "common.h"
#include "sim.h"
#include "fll.h"
#include "adc.h"
#include "pdb.h"
#include "vref.h"
#include <stdio.h>

/* local functions prototypes                                       */
void DMACH0_CH1_Init(void);
static float32_t getCpuTemp(uint16_t raw);
float32_t getMonitor12V(uint16_t raw);
float32_t getMonitor3V(uint16_t raw);

//******************************************************************************
//******************************************************************************
//**** Global varialbles *******************************************************
//******************************************************************************
//}
#define COLD_SLOPE 0.001668
#define HOT_SLOPE 0.001749
#define VTEMP25 0.7012
#define VTEMP_INV_1_2V 0.0029296875
#define VTEMP_INV_3_3V 0.0000503540
#define VTEMP_INV_3V 0.0000457763
#define VTEMP_INV_5V 0.0000762939
#define VTEMP_3V 20173.3037628242
#define BANDGAP 6585.833333
#define B1VSlope 1699.636
#define B2VSlope 4740.526042



volatile unsigned short ui_adc_result[16] = {1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0};  //Data buffer 3 x 4 samples
volatile unsigned char  uc_adc_mux[6]     = { AD26,  AD15, AD05 };                //Mux scan line //AD05 = BoardVoltage 12V  //AD15 = 3.3 V //AD26 Board Temp
#define NUM_OF_SAMPLES 4
#define NUM_OF_MUX 3
#define B1_V_OFFSET 2
#define B2_V_OFFSET 1
#define CPU_TEMP_OFFSET 0

  //****************************************************************************
  //**** ADC varialbles ********************************************************
  //****************************************************************************
tADC_CALIB              ram_ADC1;                                               //ADC calibration field


//extern "C"
//{
  /***************************************************************************//*!
 * @brief   DMA channel 1 isr - major loop comlete
 ******************************************************************************/
void DMA1_DMA17_IRQHandler(void)
{
  uint16_t i;
  uint16_t raw = 0;
  float32_t result = 0.0;
  DMA0->CINT = DMA_CINT_CINT(1);                                                  //Clear the interrupt flag for channel 0
  //****************************************************************************
  //**** ADC result buffer is full *********************************************
  //****************************************************************************
  printf("*************************************\n");
//  printf("ADC result buffer is full\n");
  for(i = 0; i < sizeof(ui_adc_result)/2; i++)
  printf("%02d  :%d\n",i, ui_adc_result[i]);

  
  
  for(i = B1_V_OFFSET; i < NUM_OF_SAMPLES*NUM_OF_MUX; i+=NUM_OF_MUX)
    raw += ui_adc_result[i];  
  result = getMonitor12V(raw/NUM_OF_SAMPLES);
  printf("12V BoardVolt: %f \n",result);
   raw = 0;
  for(i = B2_V_OFFSET; i < NUM_OF_SAMPLES*NUM_OF_MUX; i+=NUM_OF_MUX)
    raw += ui_adc_result[i];
  
  result = getMonitor3V(raw/NUM_OF_SAMPLES);
  printf("3V BoardVolt: %f \n",result);
  raw = 0;
  for(i = CPU_TEMP_OFFSET; i < NUM_OF_SAMPLES*NUM_OF_MUX; i+=NUM_OF_MUX)
    raw += ui_adc_result[i];
    
  result = getCpuTemp(raw/NUM_OF_SAMPLES);
  printf("CPU temperatur: %f \n",result);
    
  printf("*************************************\n");
}



extern void SER_Init      (int baudRate);           /* Initialize Serial Interface       */	

//******************************************************************************
//******************************************************************************
//**** Main Loop ***************************************************************
//******************************************************************************
void main(void)
{

  SER_Init(9600);
    
  printf("Hello PDB ADC DMA Test\n");
  
//  SIM->SCGC4 |= SIM_SCGC4_VREF_MASK;
//  VREF->SC |= VREF_SC_VREFEN_MASK;
//  
//  while ( ( VREF->SC & VREF_SC_VREFST_MASK ) == 0 )
//  { 

//  }             // wait bandgap to stabilize

//  VREF->SC |= VREF_SC_REGEN_MASK | VREF_SC_MODE_LV( 1 );
//  PMC->REGSC |= PMC_REGSC_BGBE_MASK; 
  
  //****************************************************************************
  //**** MCU periphery initialize **********************************************
  //****************************************************************************
  SIM_Init(SIM_MODULE_CONFIG);          
  FLL_Init(FLL_MODULE_CONFIG_FEI_96MHZ);
  //**************************************************************************
  //**** VREF config *********************************************************
  //**************************************************************************
  VREF_Init (VREF_MODULE_TIGHTREG_CONFIG);
  //**************************************************************************
  //**** PDB config **********************************************************
  //**************************************************************************
  PDBCH0TRG0_Init (PDB_PRETRG_ON_DEALYED_CONFIG, 0x0010);
  PDB_Init (PDB_MODULE_SWTRG_NO_IRQ_CONFIG( PDB_PRESCALER_32, PDB_MULT_40, 0x4000, 0xFFFF), 1, NULL);
  //**************************************************************************
  //**** ADC config **********************************************************
  //**************************************************************************
  //************************************************************************
  //**** ADC calibrations **************************************************
  //************************************************************************
  ADC_ExecCalib (ADC1, ADC_MODULE_16B_IREF_HWTRG_CONFIG_U, &ram_ADC1);
  //************************************************************************
  //**** ADC1,ADC1 initialization ******************************************
  //************************************************************************
  ADC_Init  (ADC1, ADC_MODULE_16B_HWTRG_DMA_IREF_AVG16_PGAOFF_CONFIG, ADC_CH_SE_POLL_CONFIG(AD05), ADC_CH_SE_POLL_CONFIG(AD05), 2, NULL);
  //************************************************************************
  //**** DMA channel 0 *****************************************************
  //************************************************************************
  DMACH0_CH1_Init();
  //**************************************************************************
  //**** Enable all interrupts ***********************************************
  //**************************************************************************
  __enable_irq();

  //**************************************************************************
  //**** Start PDB ***********************************************************
  //**************************************************************************
  PDB_Trigger();                                                              //Start ADC sampling   

  //****************************************************************************
  //**** Infinite Loop *********************************************************
  //****************************************************************************
  for(;;)
  {  
    
    while (!ADC_Ready (ADC1,CHA));
    

  }//End of Infinite loop
 
}//End of Main

/***************************************************************************//*!
 * @brief   DMA channel 0 channel 1 function definition
 ******************************************************************************/
void DMACH0_CH1_Init(void)
{  
  //****************************************************************************
  //**** DMA channel 0, use for Write ADC mux channel, form SRAM to ADC ********
  //****************************************************************************
  DMAMUX0->CHCFG[0]           = DMAMUX_CHCFG_ENBL_MASK|DMAMUX_CHCFG_SOURCE(54);   //DMA source DMA Mux
  DMA0->TCD[0].SADDR          = (uint32) &uc_adc_mux[0];                            //New mux setiing
  DMA0->TCD[0].SOFF           = 0x01;                                               //Source address increment, adding "1"
  DMA0->TCD[0].SLAST          = (uint32) -3;                                        //Source address decrement after major loop complete
  DMA0->TCD[0].DADDR          = (uint32) &ADC1->SC1[0];                              //Destination address ADC1 mux selector
  DMA0->TCD[0].DOFF           = 0x00;                                               //Destination address increment, adding "0"
  DMA0->TCD[0].DLAST_SGA       = 0x00;                                               //Destination address shift, go to back to [0]
  DMA0->TCD[0].NBYTES_MLNO    = 0x01;                                               //No of bytes minor loop
  DMA0->TCD[0].BITER_ELINKNO  = 0x03;                                               //Major loop step, 3 ADC channel are scaning
  DMA0->TCD[0].CITER_ELINKNO  = 0x03;                                               //Major loop step, 3 ADC channel are scaning
  DMA0->TCD[0].ATTR           = DMA_ATTR_SSIZE(0)|DMA_ATTR_DSIZE(0);                //Source a destination size, 8bit
  DMA0->TCD[0].CSR            = 0x0000;                                             //
  
  //****************************************************************************
  //**** DMA channel 1, use for Read ADC result data, form ADC to SRAM *********
  //****************************************************************************
  NVIC_EnableIRQ(DMA1_DMA17_IRQn);                                                      //17 

  DMAMUX0->CHCFG[1]           = DMAMUX_CHCFG_ENBL_MASK|DMAMUX_CHCFG_SOURCE(41);     //DMA source ADC1
  DMA0->TCD[1].SADDR          = (uint32) &ADC1->R[0];                                  //Source address ADC1 result register
  DMA0->TCD[1].SOFF           = 0x00;                                               //Source address increment, adding "0"
  DMA0->TCD[1].SLAST          = 0x00;                                               //Source address decrement after major loop complete
  DMA0->TCD[1].DADDR          = (uint32) &ui_adc_result[0];                         //Destination address, ADC1 result buffer
  DMA0->TCD[1].DOFF           = 0x02;                                               //Destination address increment, adding "+2"
  DMA0->TCD[1].DLAST_SGA       = (uint32) -24;                                       //Destination address decrement after major loop complete
  DMA0->TCD[1].NBYTES_MLNO    = 0x02;                                               //No of bytes minor loop, 16bit ADC result
  DMA0->TCD[1].BITER_ELINKNO  = (DMA_BITER_ELINKNO_ELINK_MASK|0x0000|0x0C);         //Channel 0 Link, Channel 0, Major loop step 3 x 4 = 12
  DMA0->TCD[1].CITER_ELINKNO  = (DMA_CITER_ELINKNO_ELINK_MASK|0x0C);                //Channel 0 Link, Major loop step 3 x 4 = 12
  DMA0->TCD[1].ATTR           = DMA_ATTR_SSIZE(1)|DMA_ATTR_DSIZE(1);                //Source a destination size, 16bit
  DMA0->TCD[1].CSR            = (DMA_CSR_MAJORLINKCH(0)|DMA_CSR_MAJORELINK_MASK)|   //Major loop finished start request for Channel 0
                            DMA_CSR_INTMAJOR_MASK; //|DMA_CSR_INTHALF_MASK;     //Irq. enable, full transfer, half complet transfer
  DMA0->ERQ                |= DMA_ERQ_ERQ1_MASK;                                  //HW request enable

}


static float32_t getCpuTemp(uint16_t raw)
{
    
  float32_t temp = 0.0;  
  float32_t Vtemp = (float32_t)(raw/VTEMP_3V);
  
  if(Vtemp >= VTEMP25)
  {
    //Cold Slope
    temp = 25 - ((Vtemp - VTEMP25)/ COLD_SLOPE);
  }
  else
  {
    //Hot Slope
    temp = 25 - ((Vtemp - VTEMP25)/ HOT_SLOPE);
  }
  
  return temp;
}

float32_t getMonitor12V(uint16_t raw)
{
  return raw/B1VSlope;
}

float32_t getMonitor3V(uint16_t raw)
{
  return raw/B2VSlope;
}



